# Simply Cloth Material by Vjaceslav Tissen
# vjaceslavt@gmail.com
# In Cooperation with Daniel Meier
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTIBILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


from multiprocessing import context
from random import randint
import bpy
import os
import pathlib
from pathlib import Path
from bpy.props import IntProperty, EnumProperty, BoolProperty, FloatProperty, StringProperty


class SwitchSelectMainTextures(bpy.types.Operator):
    bl_idname = "object.sc_material_switch_select_main_textures"
    bl_label = "Switch Base Texture"
    bl_description = "Switch Base Texture"

    direction: StringProperty(default="FORWARD")
    folderName: StringProperty(default='Textures-Pattern')
    path: StringProperty =  os.getcwd()
    textureType: StringProperty(default="BASE")
    simplyMaterialSlot: IntProperty(default=0)

    activeMaterialOnCurrentObject = StringProperty(default="")
    lastMaterialOnLastObject = StringProperty(default="")

    activeObject: StringProperty(default="")
    lastObject: StringProperty(default="")

    openImage: BoolProperty(default=False)
    
    rep_dir_name = Path(__file__).parent.parent.name
    addons_path = bpy.utils.user_resource(resource_type="EXTENSIONS",path=rep_dir_name)
    script_path = os.path.join(addons_path, "SimplyMaterial")

    texturePattern_dir = "/../Textures-Pattern/"
    texturePattern_dirImages = os.path.join(script_path, "Textures-Pattern")

    imageName: StringProperty(default="")
    images = []

    imageFilePath = StringProperty(default = "")

    def checkDevWorkflowBooleanAndSetTexutrePath(self, context):
        if context.active_object.scm_DEV_Workflow:
            self.texturePattern_dir = "/../Textures-Pattern/"
        elif context.active_object.scm_DEV_Workflow == False:
            self.texturePattern_dir = os.path.join(self.script_path, "Textures-Pattern")

    def getAllImagesFromDirectory(self, context):
        for path, dir, files in os.walk(self.texturePattern_dirImages):
            for filename in files:
                self.images.append(filename)
        context.active_object.scm_imageCount = len(self.images)

    def selectCorrectUseType(self, context, imageIndex):
        if self.textureType == "BASE":
            context.active_object.scm_imageBaseIndex = imageIndex
            self.imageFilePath = self.texturePattern_dir+'/'+self.images[imageIndex]
        elif self.textureType == "MICRO":
            context.active_object.scm_imageMicroIndex = imageIndex
            self.imageFilePath = self.texturePattern_dir+'/'+self.images[imageIndex]
        elif self.textureType == "OVERLAY":
            context.active_object.scm_imageOverlayIndex = imageIndex
            self.imageFilePath = self.texturePattern_dir+'/'+self.images[imageIndex]

    def switchOrOpenImageBySelectedType(self, context, textureTypeNames, material, state):
        if self.textureType == "BASE":
            if state == "SWITCH":
                material.node_tree.nodes[textureTypeNames[0]].image.filepath =  self.imageFilePath

            if state == "OPEN":
                bpy.ops.image.open(
                    filepath=self.texturePattern_dir+'/'+self.images[context.active_object.scm_imageBaseIndex],
                directory=self.texturePattern_dir+'/', 
                files=[{"name":self.images[context.active_object.scm_imageBaseIndex], "name":self.images[context.active_object.scm_imageBaseIndex]}])
                self.imageName = self.images[context.active_object.scm_imageBaseIndex]
                # material.node_tree.nodes[textureTypeNames[0]].image.filepath =  self.texturePattern_dir+"/"+self.images[context.active_object.scm_imageBaseIndex]
                material.node_tree.nodes[textureTypeNames[0]].image.filepath =  self.imageFilePath

        elif self.textureType == "MICRO":
            if state == "SWITCH":
                material.node_tree.nodes[textureTypeNames[1]].image.filepath =  self.imageFilePath
            if state == "OPEN":
                bpy.ops.image.open(
                    filepath=self.texturePattern_dir+'/'+self.images[context.active_object.scm_imageMicroIndex],
                directory=self.texturePattern_dir+'/', 
                files=[{"name":self.images[context.active_object.scm_imageMicroIndex], "name":self.images[context.active_object.scm_imageMicroIndex]}])
                self.imageName = self.images[context.active_object.scm_imageMicroIndex]
                material.node_tree.nodes[textureTypeNames[1]].image.filepath =  self.imageFilePath

        elif self.textureType == "OVERLAY":
            if state == "SWITCH":
                material.node_tree.nodes[textureTypeNames[2]].image.filepath =  self.imageFilePath
            if state == "OPEN":
                bpy.ops.image.open(
                    filepath=self.texturePattern_dir+'/'+self.images[context.active_object.scm_imageOverlayIndex],
                directory=self.texturePattern_dir+'/', 
                files=[{"name":self.images[context.active_object.scm_imageOverlayIndex], "name":self.images[context.active_object.scm_imageOverlayIndex]}])
                self.imageName = self.images[context.active_object.scm_imageOverlayIndex]
                material.node_tree.nodes[textureTypeNames[2]].image.filepath =  self.imageFilePath

    def addIndexForTextureSelect(self, context):
        obj = context.active_object
        textureTypeNames = ["SC_Base_Image", "SC_Micro_Texture","SC_Overlay_Texture"]
        textureTypeSwitch = None
        # SELECT TYPE
        if self.textureType == "BASE":
            textureTypeSwitch = textureTypeNames[0]
            imageIndex = obj.scm_imageBaseIndex
        elif self.textureType == "MICRO":
            textureTypeSwitch = textureTypeNames[1]
            imageIndex = obj.scm_imageMicroIndex
        elif self.textureType == "OVERLAY":
            textureTypeSwitch = textureTypeNames[2]
            imageIndex = obj.scm_imageOverlayIndex

        # CHANGE INDEX OF SELECTED TYPE
        if self.direction == "FORWARD":
            if imageIndex == len(self.images)-1:
                imageIndex = 0
                self.selectCorrectUseType(context,imageIndex)
            else:
               imageIndex = imageIndex +1
               self.selectCorrectUseType(context,imageIndex)
        elif self.direction == "BACKWARD":
            if imageIndex == 0:
                imageIndex = len(self.images)-1
                self.selectCorrectUseType(context,imageIndex)
            else:
                imageIndex = imageIndex -1
                self.selectCorrectUseType(context,imageIndex)

        currentMaterialName = self.activeMaterialOnCurrentObject

        # CHANGE IMAGE BY SELECTED TYPE AND INDEX
        for mat in bpy.data.materials:
            if self.activeMaterialOnCurrentObject in mat.name:
                if self.activeMaterialOnCurrentObject == mat.name:
                    if currentMaterialName in bpy.data.images:
                        state="SWITCH"
                        self.switchOrOpenImageBySelectedType(context, textureTypeNames, mat, state)
                    else:
                        state="OPEN"
                        self.switchOrOpenImageBySelectedType(context, textureTypeNames, mat, state)
                pass

    def getCurrentMaterialSlotFromActiveObject(self, context):
        for i, mat in enumerate(context.active_object.material_slots):
            if "SimplyMaterialInfo" in mat.material.node_tree.nodes:
                self.simplyMaterialSlot = i
                context.active_object.scm_materialSlot = i
                self.activeMaterialOnCurrentObject = context.active_object.material_slots[self.simplyMaterialSlot].name
                pass

    def getLastandActiveObjects(self, context):
        if self.lastObject == "" or self.activeObject == "":
            self.addActiveAndLastObject(context)
            self.getCurrentMaterialSlotFromActiveObject(context)
            self.lastMaterialOnLastObject = self.activeMaterialOnCurrentObject

        elif self.lastObject != context.active_object.name:
            self.openImage = True
            self.lastObject = context.active_object.name
            self.getCurrentMaterialSlotFromActiveObject(context)
            self.lastMaterialOnLastObject = self.activeMaterialOnCurrentObject

        elif self.lastObject == context.active_object.name:
            self.activeObject = context.active_object.name
            self.lastObject = self.activeObject
            self.getCurrentMaterialSlotFromActiveObject(context)
            self.lastMaterialOnLastObject = self.activeMaterialOnCurrentObject

    def addActiveAndLastObject(self, context):
        self.activeObject = context.active_object.name
        self.lastObject = self.activeObject
        
        self.getCurrentMaterialSlotFromActiveObject(context)

    def removeNotUsedImages(self, context):
        for image in bpy.data.images:
            if self.imageName == image.name:
                pass

    def execute(self, context):
        if len(self.images) == 0 or context.active_object.scm_imageCount == 0:
            self.getAllImagesFromDirectory(context)
        self.getLastandActiveObjects(context)
        self.checkDevWorkflowBooleanAndSetTexutrePath(context)
        self.addIndexForTextureSelect(context)
        self.removeNotUsedImages(context)
        return {'FINISHED'}

class RefreshAvailableTextures(bpy.types.Operator):
    bl_idname = "scene.sc_material_refresh_textures"
    bl_label = "Refresh Texture"
    bl_description = "Refresh Texture"
    direction: StringProperty(default="FORWARD")
    folderName: StringProperty(default='Textures-Pattern')
    path: StringProperty =  os.getcwd()
    images = []

    activeMaterialOnCurrentObject = any
    simplyMaterialSlot: IntProperty(default=0)

    def getCurrentMaterialSlotFromActiveObject(self, context):
        for i, mat in enumerate(context.active_object.material_slots):
            
            if "SimplyMaterialInfo" in mat.material.node_tree.nodes:
                self.simplyMaterialSlot = i
                context.active_object.scm_materialSlot = i

    def getAllImagesFromDirectory(self, context):
        for path, dir, files in os.walk(self.path+"/"+self.folderName):
            for filename in files:
                self.images.append(filename)
        context.active_object.scm_imageCount = len(self.images)


    def resetTexturesCount(self, context):
        context.active_object.scm_imageCount = 0
        self.images.clear()

    def fixImageNamesToActiveObject(self,context):
        activeObject = context.active_object

        for i, mat in enumerate(activeObject.material_slots):
            
            if "SimplyMaterialInfo" in mat.material.node_tree.nodes:
                self.simplyMaterialSlot = i
                context.active_object.scm_materialSlot = i

                if "SC_Base_Image" in mat.material.node_tree.nodes:
                    bpy.data.materials[self.activeMaterialOnCurrentObject].node_tree.nodes['SC_Base_Image'].image.name = self.images[context.active_object.scm_imageBaseIndex]+" | "+self.activeMaterialOnCurrentObject
                if "SC_Micro_Texture" in mat.material.node_tree.nodes:
                    bpy.data.materials[self.activeMaterialOnCurrentObject].node_tree.nodes['SC_Micro_Texture'].image.name = self.images[context.active_object.scm_imageBaseIndex]+" | "+self.activeMaterialOnCurrentObject
                if "SC_Overlay_Texture" in mat.material.node_tree.nodes:
                    bpy.data.materials[self.activeMaterialOnCurrentObject].node_tree.nodes['SC_Overlay_Texture'].image.name = self.images[context.active_object.scm_imageBaseIndex]+" | "+self.activeMaterialOnCurrentObject

    def execute(self, context):
        self.resetTexturesCount(context)
        self.getAllImagesFromDirectory(context)
        self.getCurrentMaterialSlotFromActiveObject(context)
        self.activeMaterialOnCurrentObject = context.active_object.material_slots[self.simplyMaterialSlot].name
        
        bpy.context.object.active_material.preview_render_type = 'CUBE'
        return {'FINISHED'}

class CyclesExperimentalAdaptiveSubdivisionSetup(bpy.types.Operator):
    bl_idname = "scene.scm_cycles_experimental_subdivision"
    bl_label = "Experimental Cycles Subdivision"
    bl_description = "Experimental Cycles Subdivision"

    def switchRenderEngineWithDisplacement(self, context):
        
        activeObjectMaterialName = context.active_object.material_slots[0].name
        if bpy.context.scene.render.engine == 'BLENDER_EEVEE':
            bpy.context.scene.render.engine = 'CYCLES'
            bpy.data.materials[activeObjectMaterialName].node_tree.nodes["SM_Displacement_Node"].mute = False
            # bpy.context.scene.render.engine = 'CYCLES'
            bpy.context.scene.cycles.feature_set = 'EXPERIMENTAL'
            if "Subdivision" not in context.active_object.modifiers:
                bpy.ops.object.modifier_add(type='SUBSURF')
            else:
                bpy.context.object.cycles.use_adaptive_subdivision = True
                bpy.context.scene.cycles.dicing_rate = 0.5
                bpy.context.scene.cycles.preview_dicing_rate = 6
        else:
            # bpy.context.scene.render.engine = 'BLENDER_EEVEE'
            bpy.data.materials[activeObjectMaterialName].node_tree.nodes["SM_Displacement_Node"].mute = True
            if "SUBSURF" in context.active_object.modifiers:
                bpy.context.object.modifiers["Subdivision"].show_viewport = False
                bpy.context.object.modifiers["Subdivision"].show_render = False

    def execute(self, context):
        self.switchRenderEngineWithDisplacement(context)
        return {'FINISHED'}

class CleanUpSimplyImagesFromBlendFile(bpy.types.Operator):
    bl_idname = "scene.scm_cleanup_images"
    bl_label = "Remove all Images"
    bl_description = "Remove all images"

    def removeAllImagesFromScene(self, context):
        for image in bpy.data.images:
            if "SC" or "simply" in image.name:
                if "simply_000" or "simply_001" or "simply_002" != image.name:
                    bpy.data.images.remove(image)

    def execute(self, context):
        self.removeAllImagesFromScene(context)
        return {'FINISHED'}
        
class SetUpAndRenderCavityImageTextureForMaterial(bpy.types.Operator):
    bl_idname = "scene.scm_setup_render_cavity_img_texture"
    bl_label = "render image texture for cavity"
    bl_description = "Render img for Cavity"

    simplyMaterialSlot: IntProperty(default=0)
    currentMaterialSlotName: StringProperty(default="")
    cavityImageName: StringProperty(default="")

    def createNewCavityImage(self, context):
        self.cavityImageName = "SCM_Cavity_"+context.active_object.name
        bpy.data.images.new(self.cavityImageName, 1024, 1024, alpha=False)

    def setupCyclesForRendering(self, context):
        bpy.context.scene.render.engine = "CYCLES"
        bpy.context.scene.cycles.samples = 16
        bpy.context.scene.cycles.time_limit = 10
        bpy.context.scene.render.bake.target = 'IMAGE_TEXTURES'
        bpy.context.scene.render.bake.use_pass_color = True
        bpy.context.scene.render.bake.use_pass_direct = False
        bpy.context.scene.render.bake.use_pass_indirect = False
        bpy.context.scene.cycles.bake_type = 'EMIT'
                    
    def getCurrentMaterialForCavityBaking(self, context):
        
        for i, mat in enumerate(context.active_object.material_slots):
            if "SimplyMaterialInfo" in mat.material.node_tree.nodes:
                self.simplyMaterialSlot = i
                self.currentMaterialSlotName = mat.material.name
                context.active_object.scm_materialSlot = i

                mat.material.node_tree.nodes["Material Output"].select = False
                mat.material.node_tree.nodes["SCM_Cavity_Emissive_Output"].select = True
                mat.material.node_tree.nodes["SCM_Emission"].mute = False
                self.currentMaterial.node_tree.nodes["SM_Cavity_Emissive_Switch"].outputs[0].default_value = 0

                mat.material.node_tree.nodes["SCM_Wear_ImageTexture"].select = True
                mat.material.node_tree.nodes["SCM_Wear_ImageTexture"].image = bpy.data.images[self.cavityImageName]
                pass

    def renderCavityImage(self, context):
        bpy.ops.object.bake(type='EMIT')

    def selectDefaultMaterialOutput(self, context):
        context.active_object.material_slots[self.simplyMaterialSlot].material.node_tree.nodes["SCM_Emission"].mute = False
        context.active_object.material_slots[self.simplyMaterialSlot].material.node_tree.nodes["Material Output"].mute = False
        context.active_object.material_slots[self.simplyMaterialSlot].material.node_tree.nodes["Material Output"].select = True
        context.scene.scm_UI_PreviewForCavityBake = False

    def execute(self, context):
        self.createNewCavityImage(context)
        self.setupCyclesForRendering(context)
        self.getCurrentMaterialForCavityBaking(context)
        self.renderCavityImage(context)
        self.selectDefaultMaterialOutput(context)

        return {'FINISHED'}      

class SetupPreviewForCavityBaking(bpy.types.Operator):
    bl_idname = "scene.sm_preview_cavity_bake"
    bl_label = "render image texture for cavity"
    bl_description = "Render img for Cavity"

    simplyMaterialSlot: IntProperty(default=0)
    currentMaterialSlotName: StringProperty(default="")
    
    def setRenderingForCycles(self, context):
        bpy.context.scene.render.engine = 'CYCLES'
        bpy.context.space_data.shading.type = 'RENDERED'
    def changeMaterialBack(self, context):
        context.active_object.material_slots[self.simplyMaterialSlot].material.node_tree.nodes["SCM_Cavity_Emissive_Output"].mute =True
        context.active_object.material_slots[self.simplyMaterialSlot].material.node_tree.nodes["SCM_Cavity_Emissive_Output"].select =False
        context.active_object.material_slots[self.simplyMaterialSlot].material.node_tree.nodes["SCM_Cavity_Emissive_Output"].is_active_output = False
        context.active_object.material_slots[self.simplyMaterialSlot].material.node_tree.nodes["SCM_Emission"].mute = True
        context.active_object.material_slots[self.simplyMaterialSlot].material.node_tree.nodes["Material Output"].mute = False
        context.active_object.material_slots[self.simplyMaterialSlot].material.node_tree.nodes["Material Output"].select = True
        context.active_object.material_slots[self.simplyMaterialSlot].material.node_tree.nodes["Material Output"].is_active_output = True
        context.active_object.material_slots[self.simplyMaterialSlot].material.node_tree.nodes["SM_Cavity_Emissive_Switch"].outputs[0].default_value = 1
    def changeMaterialForCavityBake(self, context):
        context.active_object.material_slots[self.simplyMaterialSlot].material.node_tree.nodes["Material Output"].select = False
        context.active_object.material_slots[self.simplyMaterialSlot].material.node_tree.nodes["Material Output"].mute = True
        context.active_object.material_slots[self.simplyMaterialSlot].material.node_tree.nodes["SCM_Emission"].mute = False
        context.active_object.material_slots[self.simplyMaterialSlot].material.node_tree.nodes["SCM_Cavity_Emissive_Output"].mute = False
        context.active_object.material_slots[self.simplyMaterialSlot].material.node_tree.nodes["SCM_Cavity_Emissive_Output"].select =True
        context.active_object.material_slots[self.simplyMaterialSlot].material.node_tree.nodes["SCM_Cavity_Emissive_Output"].is_active_output = True
        context.active_object.material_slots[self.simplyMaterialSlot].material.node_tree.nodes["SM_Cavity_Emissive_Switch"].outputs[0].default_value = 0
        
    def execute(self, context):
        if context.scene.scm_UI_PreviewForCavityBake == True:
            context.scene.scm_UI_PreviewForCavityBake = False
        elif context.scene.scm_UI_PreviewForCavityBake == False:
            context.scene.scm_UI_PreviewForCavityBake = True
        print(context.scene.scm_UI_PreviewForCavityBake)

        if context.scene.scm_UI_PreviewForCavityBake == True:
            self.setRenderingForCycles(context)
            self.changeMaterialForCavityBake(context)

        elif context.scene.scm_UI_PreviewForCavityBake == False:
            self.changeMaterialBack(context)

        return {'FINISHED'}

class SetUpMaterialForActiveObject(bpy.types.Operator):
    bl_idname = "object.sm_setup_material_for_active_object"
    bl_label = "Setup Material for active object"
    bl_description = "Setup Material for active object"
    activeObjectMaterialSlot = IntProperty(default= 0)
    activeObjectMaterialName = StringProperty(default="")

    folderName: StringProperty(default='Textures-Pattern')
    blendFilesFolder: StringProperty(default='Blend Files')
    materialBlendFile: StringProperty(default='Simply_Mat_Master.blend')

    path: StringProperty =  os.getcwd()

    rep_dir_name = Path(__file__).parent.parent.name
    addons_path = bpy.utils.user_resource(resource_type="EXTENSIONS",path=rep_dir_name)

    script_path = os.path.join(addons_path, "SimplyMaterial")
    blendFiles_dir = os.path.join(script_path, "Blend Files")
    materialBlendFilePath = os.path.join(blendFiles_dir, "Simply_Mat_Master.blend")

    texturePattern_dir = os.path.join(script_path, "Textures-Pattern")
    images = []

    def getAllImagesFromDirectory(self, context):
        for path, dir, files in os.walk(self.texturePattern_dir):
            for filename in files:
                self.images.append(filename)
        context.active_object.scm_imageCount = len(self.images)

    def getSimplyMaterialFromActiveObject(self, context):
        obj = context.active_object
        
        for i, mat in enumerate(obj.material_slots):
            if "SimplyMaterialInfo" in mat.material.node_tree.nodes:
                self.activeObjectMaterialName = mat.name
                self.activeObjectMaterialSlot = i

    # SETUP TEXTURES
    def setupImageNamesInSimplyMaterialOnActiveObject(self, context):
        simplyMaterial = context.active_object.material_slots[self.activeObjectMaterialSlot].material
        
        if bpy.data.images:
            for image in bpy.data.images:
                if "Base" in image.name:
                    simplyStandardbase = "simply_000.png"
                    bpy.ops.image.open(filepath=self.texturePattern_dir+'/'+simplyStandardbase,directory=self.texturePattern_dir+'/', files=[{"name":simplyStandardbase, "name":simplyStandardbase}])
                    print(simplyStandardbase)

                if "Micro" in image.name:
                    simplyStandardMicro = "simply_001.png"
                    bpy.ops.image.open(filepath=self.texturePattern_dir+'/'+simplyStandardMicro,directory=self.texturePattern_dir+'/', files=[{"name":simplyStandardMicro, "name":simplyStandardMicro}])
                    print(simplyStandardMicro)

                if "Overlay" in image.name:
                    simplyStandardOverlay = "simply_overlay_001.png"
                    bpy.ops.image.open(filepath=self.texturePattern_dir+'/'+simplyStandardOverlay,directory=self.texturePattern_dir+'/', files=[{"name":simplyStandardOverlay, "name":simplyStandardOverlay}])
                    print(simplyStandardOverlay)
        else:
            simplyStandardbase = "simply_000.png"
            simplyStandardMicro = "simply_001.png"
            simplyStandardOverlay = "simply_overlay_001.png"


        if "Base" or "Micro" or "Overlay" in bpy.data.images:
            simplyStandardbaseRandom = self.images[randint(3,len(self.images))]
            simplyStandardMicroRandom = self.images[randint(4,len(self.images))]
            simplyStandardOverlayRandom = "simply_overlay_001.png"

        tag = ".png"
        for node in simplyMaterial.node_tree.nodes:
            if node.label  == "SC_Base_Image":
                bpy.ops.image.open(filepath=self.texturePattern_dir+'/'+simplyStandardbaseRandom,directory=self.texturePattern_dir+'/', files=[{"name":simplyStandardbaseRandom, "name":simplyStandardbaseRandom}])

                node.image.filepath = self.texturePattern_dir+'/'+simplyStandardbaseRandom
                simplyStandardbaseRandom.replace(tag, "")
                node.image.name = "Base Texture - "+ self.activeObjectMaterialName

            if node.label  == "SC_Micro_Texture":
                bpy.ops.image.open(filepath=self.texturePattern_dir+'/'+simplyStandardMicroRandom,directory=self.texturePattern_dir+'/', files=[{"name":simplyStandardMicroRandom, "name":simplyStandardMicroRandom}])

                node.image.filepath = self.texturePattern_dir+'/'+simplyStandardMicroRandom
                simplyStandardMicroRandom.replace(tag, "")
                node.image.name = "Micro Texture" + " - " + self.activeObjectMaterialName

            if node.label  == "SC_Overlay_Texture":
                bpy.ops.image.open(filepath=self.texturePattern_dir+'/'+simplyStandardOverlayRandom,directory=self.texturePattern_dir+'/', files=[{"name":simplyStandardOverlayRandom, "name":simplyStandardOverlayRandom}])

                node.image.filepath = self.texturePattern_dir+'/'+simplyStandardOverlayRandom
                simplyStandardOverlayRandom.replace(tag, "")
                node.image.name = "Overlay Texture" + " - " + self.activeObjectMaterialName

    def appendSimplyMaterialFromBlendFile(self, context):       

        
        # your code: materialdirectory = self.materialBlendFilePath+"/Material/"
        materialdirectory = self.materialBlendFilePath+"/Material/"
        materialFilename="SC_Master"
        # your code: materialFilePath = materialdirectory
        materialFilePath = os.path.join(materialdirectory, materialFilename)
        test = ""

        bpy.ops.wm.append(filepath= materialFilePath, directory=materialdirectory, filename="SC_Master")
        newMaterialObjectName = "Simply Material | " + context.active_object.name
        bpy.data.materials[materialFilename].name = newMaterialObjectName
        self.addMaterialToObjectPolygons(context, context.active_object, newMaterialObjectName)

    def addMaterialToObjectPolygons(self, context, obj, matName):
        obj = bpy.context.active_object
        bpy.data.objects[obj.name].active_material = bpy.data.materials[matName]

    def removeNotNeededMaterial(self, context):
        material_slots = context.active_object.material_slots
        for material in material_slots:
            bpy.ops.object.material_slot_remove()

    def execute(self, context):
        obj = bpy.context.active_object
        if bpy.data.objects[obj.name].active_material is not None:
            self.removeNotNeededMaterial(context)

        if bpy.data.objects[obj.name].active_material is None:
            self.appendSimplyMaterialFromBlendFile(context)

        if len(self.images) == 0 or context.active_object.scm_imageCount == 0:
            self.getAllImagesFromDirectory(context)
        self.getSimplyMaterialFromActiveObject(context)
        return {'FINISHED'}

class OpenImageBySelector(bpy.types.Operator):
    bl_idname = "scene.scm_open_image_selector"
    bl_label = "Select Image by selection from filepath"
    bl_description = ""
    bl_options = {'REGISTER'}
    imageType: StringProperty(default="BASE")
    filepath: StringProperty(default="")
    nodeTree = any
    simplyMaterialSlot: IntProperty(default=0)
    currentFilePath: StringProperty(default="")

    def addNeededContext(self, context):
        self.simplyMaterialNode = context.active_object.material[self.simplyMaterialSlot].node_tree.nodes

    def getCurrentMaterialSlotFromActiveObject(self, context):
        for i, mat in enumerate(context.active_object.material_slots):
            
            if "SimplyMaterialInfo" in mat.material.node_tree.nodes:
                self.simplyMaterialSlot = i
                context.active_object.scm_materialSlot = i

    def getCurrentImage(self, context):
        if self.imageType == "BASE":
            self.currentFilePath = self.simplyMaterialNode["SC_Base_Image"].image.filepath

    def execute(self, context):
        self.addNeededContext(context)
        self.getCurrentMaterialSlotFromActiveObject(context)
        self.getCurrentImage(context)

        return{"FINISHED"}

class BakeOutputTextures(bpy.types.Operator):
    bl_idname = "object.scm_bake_output_textures"
    bl_label = "Texture Bake from active object"
    bl_description = "Bake selected Textures to Export Path"
    bl_options = {'REGISTER'}
    imageType: StringProperty(default="BASE")
    filepath: StringProperty(default="")
    nodeTree = any

    simplyMaterialSlot: IntProperty(default=0)

    currentFilePath: StringProperty(default="")
    neededTexturesToBake = any

    baseColor: StringProperty(default="BC_")
    roughness: StringProperty(default="RGH_")
    ambientOcclusion: StringProperty(default="AO_")
    normalGL: StringProperty(default="NRM-OGL_")
    normalUE: StringProperty(default="NRM-DX_")
    emissive: StringProperty(default="EMS_")
    alpha: StringProperty(default="ALPHA_")
    cavity: StringProperty(default="CAV_") 
    uv: StringProperty(default="UV_") 

    textureBatch = any

    tempBakeImageName : StringProperty(default="BC_")
    imgTempName : StringProperty(default="Bake")

    currentMaterial = any
    def getCurrentMaterialForTextureBaking(self, context):
        
        for i, mat in enumerate(context.active_object.material_slots):
            if "SimplyMaterialInfo" in mat.material.node_tree.nodes:
                self.simplyMaterialSlot = i
                self.currentMaterialSlotName = mat.material.name
                context.active_object.scm_materialSlot = i

                bpy.context.area.ui_type = 'ShaderNodeTree'
                self.currentMaterial= mat.material
                mat.material.node_tree.nodes["SCM_Bake_Texture_Image"].select = True
                mat.material.node_tree.nodes["SCM_Bake_Texture_Image"].image = bpy.data.images[self.imgTempName]
                
                bpy.context.area.ui_type = 'VIEW_3D'
                print(self.currentMaterial)

    def setSettingsForCycles(self, context):
        bpy.context.scene.render.engine = "CYCLES"
        bpy.context.scene.cycles.samples = 12
        bpy.context.scene.cycles.time_limit = 6
        bpy.context.scene.render.bake.target = 'IMAGE_TEXTURES'
        bpy.context.scene.render.bake.use_pass_color = True
        bpy.context.scene.render.bake.use_pass_direct = False
        bpy.context.scene.render.bake.use_pass_indirect = False
        bpy.context.scene.render.bake.use_clear = True

    def createNewTempImage(self, context, type):
        self.tempBakeImageName = type+ context.active_object.scm_image_output_name
        alpha = False
        if type == self.alpha:
            alpha=True
        else:
            alpha=False
        bpy.data.images.new(self.tempBakeImageName, context.active_object.scm_image_output_width, context.active_object.scm_image_output_height, alpha=alpha,is_data = True ,)
        if type == self.baseColor:
            bpy.data.images[self.tempBakeImageName].colorspace_settings.name = 'Non-Color'
    def renderSelectedTextures(self, context):
        obj = bpy.context.active_object 
        renderPath = bpy.context.scene.render.filepath  

        self.textureBatch = [
            [context.active_object.scm_UI_Bake_Diffuse, self.baseColor],
            [context.active_object.scm_UI_Bake_Roughness, self.roughness],
            [context.active_object.scm_UI_Bake_AO, self.ambientOcclusion],
            [context.active_object.scm_UI_Bake_Normal, self.normalGL],
            [context.active_object.scm_UI_Bake_Normal_UE, self.normalUE],
            [context.active_object.scm_UI_Bake_Emissive, self.emissive],
            [context.active_object.scm_UI_Bake_Alpha, self.alpha],
            [context.active_object.scm_UI_Bake_Cavity, self.cavity],
            [context.active_object.scm_UI_Bake_UV, self.uv],
        ]
        print(self.textureBatch)
        for texture in self.textureBatch:
            if texture[0]==True:
                imgFilePathName = renderPath+texture[1]+context.active_object.scm_image_output_name+"."+context.scene.render.image_settings.file_format
                self.imgTempName= texture[1]+context.active_object.scm_image_output_name
                bpy.context.scene.render.filepath = imgFilePathName
                filepath = bpy.context.scene.render.filepath
                filepath = bpy.path.abspath(filepath)
                type = ""
                if texture[1] == self.baseColor:
                    bpy.context.scene.cycles.bake_type = 'DIFFUSE'
                    bpy.context.scene.render.bake.use_pass_color = True
                    bpy.context.scene.render.bake.use_pass_direct = False
                    bpy.context.scene.render.bake.use_pass_indirect = False

                    context.scene.render.image_settings.color_mode = 'RGB'
                    
                if texture[1] == self.alpha:
                    bpy.context.scene.cycles.bake_type = 'TRANSMISSION'
                    # bpy.context.scene.cycles.bake_type = 'EMIT'
                    bpy.context.scene.render.film_transparent = True
                    bpy.context.scene.render.bake.use_pass_direct = False
                    bpy.context.scene.render.bake.use_pass_indirect = False
                    bpy.context.scene.render.bake.use_pass_color = True

                if texture[1] == self.roughness:
                    bpy.context.scene.cycles.bake_type = 'ROUGHNESS'

                if texture[1] == self.emissive:
                    bpy.context.scene.cycles.bake_type = 'EMIT'

                if texture[1] == self.cavity:
                    bpy.context.scene.cycles.bake_type = 'EMIT'

                if texture[1] == self.ambientOcclusion:
                    bpy.context.scene.cycles.bake_type = 'AO'

                if texture[1] == self.uv:
                    bpy.context.scene.cycles.bake_type = 'UV'

                if texture[1] == self.normalGL:
                    bpy.context.scene.cycles.bake_type = 'NORMAL'        
                    bpy.context.scene.render.bake.normal_r = 'POS_X'        
                    bpy.context.scene.render.bake.normal_g = 'POS_Y'
                    bpy.context.scene.render.bake.normal_b = 'POS_Z'
                    bpy.context.scene.render.bake.normal_space = 'TANGENT'

                if texture[1] == self.normalUE:
                    bpy.context.scene.cycles.bake_type = 'NORMAL'
                    bpy.context.scene.render.bake.normal_r = 'POS_X'
                    bpy.context.scene.render.bake.normal_g = 'NEG_Y'
                    bpy.context.scene.render.bake.normal_b = 'POS_Z'
                    bpy.context.scene.render.bake.normal_space = 'TANGENT'

                self.createNewTempImage(context, type=texture[1])
                print("CREATED NEW IMAGE")
                self.getCurrentMaterialForTextureBaking(context)
                if texture[1] == self.alpha:
                    self.currentMaterial.node_tree.nodes["SCM_Shader_Alpha_Bake"].mute = False
                    self.currentMaterial.node_tree.nodes["SCM_Alpha_Output"].select = True
                    self.currentMaterial.node_tree.nodes["SCM_Alpha_Output"].is_active_output = True
                    self.currentMaterial.node_tree.nodes["Material Output"].select = False
                    if context.scene.render.image_settings.file_format != "JPEG":
                        context.scene.render.image_settings.color_mode = 'RGBA'
                else:
                    context.scene.render.image_settings.color_mode = 'RGB'

                if texture[1] == self.emissive:
                    self.currentMaterial.node_tree.nodes["SCM_Cavity_Emissive_Output"].select = True
                    self.currentMaterial.node_tree.nodes["SCM_Cavity_Emissive_Output"].is_active_output = True
                    self.currentMaterial.node_tree.nodes["SCM_Emission"].mute = False
                    self.currentMaterial.node_tree.nodes["Material Output"].select = False
                    self.currentMaterial.node_tree.nodes["Material Output"].mute = True
                    self.currentMaterial.node_tree.nodes["SM_Cavity_Emissive_Switch"].outputs[0].default_value = 1

                    bpy.data.images[self.tempBakeImageName].colorspace_settings.name = 'Linear'
                    if context.scene.render.image_settings.file_format != "JPEG":
                        bpy.context.scene.render.image_settings.color_mode = 'RGBA'
                        bpy.context.scene.render.image_settings.color_depth = '16'

                if texture[1] == self.cavity:
                    self.currentMaterial.node_tree.nodes["SCM_Cavity_Emissive_Output"].select = True
                    self.currentMaterial.node_tree.nodes["SCM_Cavity_Emissive_Output"].is_active_output = True
                    self.currentMaterial.node_tree.nodes["SCM_Emission"].mute = False
                    self.currentMaterial.node_tree.nodes["Material Output"].select = False
                    self.currentMaterial.node_tree.nodes["Material Output"].mute = True
                    self.currentMaterial.node_tree.nodes["SM_Cavity_Emissive_Switch"].outputs[0].default_value = 0

                if texture[1] == self.baseColor:
                    self.currentMaterial.node_tree.nodes["SCM_Mix_Velvet"].mute = True
                    self.currentMaterial.node_tree.nodes["SCM_Mix_Dispersion"].mute = True
                    self.currentMaterial.node_tree.nodes["SM_Glitter_Intensity"].outputs[0].default_value = 0
                    self.currentMaterial.node_tree.nodes["SM_Principled_BSDF"].inputs[17].default_value = 0

                    self.currentMaterial.node_tree.nodes["Material Output"].is_active_output = True
                    self.currentMaterial.node_tree.nodes["Material Output"].mute = False


                    bpy.data.images[self.tempBakeImageName].colorspace_settings.name = 'Linear'
                    if context.scene.render.image_settings.file_format != "JPEG" and context.scene.render.image_settings.file_format != "BMP":
                        bpy.context.scene.render.image_settings.color_mode = 'RGBA'
                        bpy.context.scene.render.image_settings.color_depth = '16'

                if texture[1] == self.cavity:
                    bpy.data.images[self.tempBakeImageName].colorspace_settings.name = 'sRGB'
                    bpy.context.scene.render.image_settings.color_mode = 'RGB'
                    if context.scene.render.image_settings.file_format != "JPEG":
                        bpy.context.scene.render.image_settings.color_depth = '16'

                if texture[1] == self.normalGL or texture[1] == self.normalUE:
                    bpy.data.images[self.tempBakeImageName].colorspace_settings.name = 'sRGB'
                    bpy.context.scene.render.image_settings.color_mode = 'RGB'
                    if context.scene.render.image_settings.file_format != "JPEG" and context.scene.render.image_settings.file_format != "BMP":
                        bpy.context.scene.render.image_settings.color_depth = '16'

                tempFresnel = self.currentMaterial.node_tree.nodes["SM_Sheen_Fresnel_Value"].outputs[0].default_value
                self.currentMaterial.node_tree.nodes["SM_Sheen_Fresnel_Value"].outputs[0].default_value = 0

                
                bpy.ops.object.bake(type= context.scene.cycles.bake_type)

                bpy.data.images[self.tempBakeImageName].pack()
                bpy.data.images[self.tempBakeImageName].source = 'FILE'
                if self.tempBakeImageName in bpy.data.images:
                    # if self.tempBakeImageName:
                    # bpy.context.area.ui_type = 'IMAGE_EDITOR'
                    # bpy.ops.image.save()
                    # bpy.context.area.ui_type = 'VIEW_3D'
                    # if self.tempBakeImageName == bpy.data.images["normal_texture"]:
                    print(self.tempBakeImageName)
                    # if bpy.context.object.scm_UI_normal_custom_active == False:
                    # if bpy.data.images[self.tempBakeImageName].filepath =="":
                    bpy.data.images.new(name=self.tempBakeImageName, width=context.object.scm_image_output_width, height= context.object.scm_image_output_height)
                    bpy.data.images[self.tempBakeImageName].filepath = filepath
                    bpy.data.images[self.tempBakeImageName].save_render(filepath=filepath)
                # print("SAVED IMAGE")

                # RESTORE SETTINGS
                self.currentMaterial.node_tree.nodes["SCM_Emission"].inputs[1].default_value = 1
                self.imgTempName = ""

                if texture[1] == self.alpha:
                    self.currentMaterial.node_tree.nodes["SCM_Shader_Alpha_Bake"].mute = True
                    self.currentMaterial.node_tree.nodes["SCM_Alpha_Output"].select = False
                    self.currentMaterial.node_tree.nodes["Material Output"].select = True
                    self.currentMaterial.node_tree.nodes["Material Output"].mute = False
                if texture[1] == self.emissive:
                    self.currentMaterial.node_tree.nodes["SCM_Emission"].mute = True
                    self.currentMaterial.node_tree.nodes["SCM_Cavity_Emissive_Output"].select = False
                    self.currentMaterial.node_tree.nodes["Material Output"].select = True
                    self.currentMaterial.node_tree.nodes["Material Output"].mute = False

                if texture[1] == self.cavity:
                    self.currentMaterial.node_tree.nodes["SCM_Emission"].mute = True
                    self.currentMaterial.node_tree.nodes["SCM_Cavity_Emissive_Output"].select = False
                    self.currentMaterial.node_tree.nodes["Material Output"].select = True
                    self.currentMaterial.node_tree.nodes["Material Output"].mute = False

                self.currentMaterial.node_tree.nodes["SM_Sheen_Fresnel_Value"].outputs[0].default_value = tempFresnel
                self.currentMaterial.node_tree.nodes["Material Output"].is_active_output = True
                self.currentMaterial.node_tree.nodes["Material Output"].mute = False
                bpy.context.scene.render.filepath = renderPath

    def execute(self, context):
        self.setSettingsForCycles(context)
        self.renderSelectedTextures(context)

        return{"FINISHED"}

class SwitchToPaintModeAndActivateCorrectImageTexture(bpy.types.Operator):
    bl_idname = "object.scm_switch_overlay_texture_paint"
    bl_label = "Texture Bake from active object"
    bl_description = "Paint Mask for Overlay Texture"
    bl_options = {'REGISTER'}

    material = any

    def getCurrentMaterial(self, context):
        for i, mat in enumerate(context.active_object.material_slots):
            if "SimplyMaterialInfo" in mat.material.node_tree.nodes:
                self.simplyMaterialSlot = i
                self.material = mat.material

    def switchToTexturePaintMode(self, context):
        bpy.context.area.ui_type = 'ShaderNodeTree'
        bpy.ops.node.select_all(action='DESELECT')

        self.material.node_tree.nodes["SCM_Overlay_Mask"].mute = False
        self.material.node_tree.nodes["SCM_Overlay_Mask"].select = True

        bpy.context.area.ui_type = 'VIEW_3D'
        bpy.ops.paint.texture_paint_toggle()
        bpy.context.scene.tool_settings.image_paint.mode = 'IMAGE'
        bpy.context.scene.tool_settings.image_paint.canvas = bpy.data.images['SCM_Overlay_Mask']
        bpy.context.space_data.shading.type = 'MATERIAL'

    def execute(self, context):
        self.getCurrentMaterial(context)
        self.switchToTexturePaintMode(context)
        return{"FINISHED"}

class SwitchToPaintModeAndActivateCorrectTearingImageTexture(bpy.types.Operator):
    bl_idname = "object.scm_switch_tear_texture_paint"
    bl_label = "Texture Bake from active object"
    bl_description = ""
    bl_options = {'REGISTER'}

    material = any
    def getCurrentMaterial(self, context):
        for i, mat in enumerate(context.active_object.material_slots):
            if "SimplyMaterialInfo" in mat.material.node_tree.nodes:
                self.simplyMaterialSlot = i
                self.material = mat.material

    def switchToTexturePaintMode(self, context):
        bpy.context.area.ui_type = 'ShaderNodeTree'
        bpy.ops.node.select_all(action='DESELECT')
        self.material.node_tree.nodes["SCM_Tear_Paint_Texture"].mute = False
        self.material.node_tree.nodes["SCM_Tear_Paint_Texture"].select = True
        bpy.context.area.ui_type = 'VIEW_3D'

        bpy.ops.paint.texture_paint_toggle()
        bpy.context.scene.tool_settings.image_paint.mode = 'IMAGE'
        bpy.context.scene.tool_settings.image_paint.canvas = bpy.data.images['SCM_Tearing_Alpha_Paint']
        bpy.context.space_data.shading.type = 'MATERIAL'

    def execute(self, context):
        self.getCurrentMaterial(context)
        self.switchToTexturePaintMode(context)
        return{"FINISHED"}

class SwitchToPaintModeAndActivateCustomBaseColorImageTexture(bpy.types.Operator):
    bl_idname = "object.scm_switch_custom_basecolor_texture_paint"
    bl_label = "Texture Paint"
    bl_description = "Switch to Texture Paint for custom Base Color"
    bl_options = {'REGISTER'}
    mode: StringProperty(default="",options={'HIDDEN'})
    material = any

    customBaseColorBlendTextureName: StringProperty(name="Texture Name: ",default="Custom Base Color Blend Texture")
    customTextureWidth: IntProperty(name="Width ", default=2048)
    customTextureHeight: IntProperty(name="Height ", default=2048)
    customTextureAlpha: BoolProperty(name="Alpha ", default=False)
    
    @classmethod
    def poll(cls, context):
        return context.active_object is not None

    def invoke(self, context, event):
        if self.mode == "CREATE":
            if self.customBaseColorBlendTextureName not in bpy.data.images:
                return context.window_manager.invoke_props_dialog(self, width=400)		
            else:
                bpy.context.area.ui_type = 'IMAGE_EDITOR'
        elif self.mode == "REMOVE":
            self.execute(context)
        elif self.mode == "SAVEIMAGE":
            self.execute(context)
        elif self.mode == "SWITCH":
            bpy.ops.paint.texture_paint_toggle()
        return {'CANCELLED'}
    def getCurrentMaterial(self, context):
        for i, mat in enumerate(context.active_object.material_slots):
            if "SimplyMaterialInfo" in mat.material.node_tree.nodes:
                self.simplyMaterialSlot = i
                self.material = mat.material

    def switchToTexturePaintMode(self, context):
        bpy.context.area.ui_type = 'ShaderNodeTree'
        bpy.ops.node.select_all(action='DESELECT')
        self.material.node_tree.nodes["SC_Base_Color_CustomTextureMix_Texture"].mute = False
        self.material.node_tree.nodes["SC_Base_Color_CustomTextureMix_Texture"].select = True
        bpy.context.area.ui_type = 'VIEW_3D'

        bpy.ops.paint.texture_paint_toggle()
        bpy.context.scene.tool_settings.image_paint.mode = 'IMAGE'
        
        bpy.context.scene.tool_settings.image_paint.canvas = bpy.data.images[self.customBaseColorBlendTextureName]
        bpy.context.space_data.shading.color_type = 'TEXTURE'

    def createNewCustomBaseColorBlendImage(self, context):
        bpy.data.images.new(name=self.customBaseColorBlendTextureName, width=self.customTextureWidth, height= self.customTextureHeight,alpha= self.customTextureAlpha)
        context.active_object.scm_image_custom_base_color_name = self.customBaseColorBlendTextureName

    def selectCustomImageInShaderNodes(self, context):
        self.material.node_tree.nodes["SC_Base_Color_CustomTextureMix_Texture"].image = bpy.data.images[self.customBaseColorBlendTextureName]
        bpy.data.images[self.customBaseColorBlendTextureName].generated_color = (1, 1, 1, 1)        

    def removeCustomBaseColorTextureImage(self, context):
        # bpy.context.area.ui_type = 'ShaderNodeTree'
        self.customBaseColorBlendTextureName = context.active_object.scm_image_custom_base_color_name
        bpy.data.images.remove(bpy.data.images[self.customBaseColorBlendTextureName])
        self.material.node_tree.nodes["SC_Base_Color_CustomTextureMix_Intensity"].outputs[0].default_value = 0


    def saveImageToSelectedFilepath(self, context):
            self.customBaseColorBlendTextureName = context.active_object.scm_image_custom_base_color_name
            bpy.context.area.ui_type = 'IMAGE_EDITOR'
            for area in context.screen.areas:
                if area.type == "IMAGE_EDITOR":
                    override = context.copy()
                    override["area"] = area
                    area.spaces.active.image = bpy.data.images[self.customBaseColorBlendTextureName]

    def execute(self, context):
        self.getCurrentMaterial(context)
        if self.mode =="SAVEIMAGE":
            self.saveImageToSelectedFilepath(context)
        if self.mode=="CREATE":
            if self.customBaseColorBlendTextureName not in bpy.data.images:
                self.createNewCustomBaseColorBlendImage(context)
                self.selectCustomImageInShaderNodes(context)
                # self.saveImageToSelectedFilepath(context)
                # self.switchToTexturePaintMode(context)
                # bpy.context.space_data.shading.type = 'MATERIAL'
                # bpy.context.space_data.shading.type = 'SOLID'
                bpy.context.space_data.shading.color_type = 'TEXTURE'
            else:
                self.switchToTexturePaintMode(context)
        if self.mode=="REMOVE":
            self.removeCustomBaseColorTextureImage(context)
        if self.mode == "SWITCH":
            self.switchToTexturePaintMode(context)


        return{"FINISHED"}

class SetupSimplyFuzzleGeometryNodes(bpy.types.Operator):
    bl_idname = "object.sm_setup_fuzzle_geo_nodes"
    bl_label = "Setup Fuzzle Geometry Nodes for active object"
    bl_description = "Setup Material for active object"

    folderName: StringProperty(default='Textures-Pattern')
    blendFilesFolder: StringProperty(default='Blend Files')
    materialBlendFile: StringProperty(default='Simply_Mat_Master.blend')

    path: StringProperty =  os.getcwd()

    rep_dir_name = Path(__file__).parent.parent.name
    addons_path = bpy.utils.user_resource(resource_type="EXTENSIONS",path=rep_dir_name)
    script_path = os.path.join(addons_path, "SimplyMaterial")
    blendFiles_dir = os.path.join(script_path, "Blend Files")
    materialBlendFilePath = os.path.join(blendFiles_dir, "Simply_Mat_Master.blend")


    def appendSimplyFuzzleGeoNodesFromBlendFile(self, context):       
        
        materialdirectory = self.materialBlendFilePath+"/NodeTree/"
        materialFilename="SM_Fuzz_Geo_Nodes"
        materialFilePath = materialdirectory
        test = ""

        bpy.ops.wm.append(filepath= materialFilePath, directory=materialdirectory, filename="SM_Fuzz_Geo_Nodes")

    def addGeometryNodesModifierToSelectedObject(self, context):
        obj = bpy.context.active_object
        modifiername = "SimplyGeoNode"+"_"+obj.name
        bpy.ops.object.modifier_add(type='NODES')
        obj.modifiers['GeometryNodes'].name = modifiername
        obj.modifiers[modifiername].node_group = bpy.data.node_groups["SM_Fuzz_Geo_Nodes"]
        obj.scm_geoNodes_fuzzle_modifier_name = modifiername

    def addNeededGeometryNodeNameParametersToObject(self, context):
        obj = bpy.context.active_object
        modifiername = "SimplyGeoNode"+"_"+obj.name
        bpy.ops.object.modifier_add(type='NODES')
        bpy.context.active_object.modifiers['GeometryNodes'].name = modifiername
        bpy.context.active_object.modifiers[modifiername].node_group = bpy.data.node_groups["SM_Fuzz_Geo_Nodes"]

    
    def execute(self, context):
        if "SM_Fuzz_Geo_Nodes" not in bpy.data.node_groups:
            self.appendSimplyFuzzleGeoNodesFromBlendFile(context)

        self.addGeometryNodesModifierToSelectedObject(context)
        context.active_object.scm_Fuzzle_Modifier = True
        return {'FINISHED'}

class SetupModifierForDisplacementUsage(bpy.types.Operator):
    bl_idname = "object.setup_modifier_for_displacement"
    bl_label = "Setup Fuzzle Geometry Nodes for active object"
    bl_description = "Setup Material for active object"

    def addSubdivisionModifier(self, context):       
        bpy.ops.object.modifier_add(type='SUBSURF')
        bpy.context.object.cycles.use_adaptive_subdivision = True

    def setupCyclesForDisplacement(self, context):
        bpy.context.scene.render.engine = 'CYCLES'
        bpy.context.scene.cycles.feature_set = 'EXPERIMENTAL'

        
    def execute(self, context):
        self.setupCyclesForDisplacement(context)
        self.addSubdivisionModifier(context)

        return {'FINISHED'}

class OpenURL(bpy.types.Operator):
	bl_idname = "scene.open_urls"
	bl_label = "Simply Support"
	bl_description = "Simply Support"
	mode:StringProperty(default="STANDARD")

	def openURL(self, context):
		bpy.ops.wm.url_open(url="https://www.youtube.com/watch?v=RrYNnrUxwIo&list=PLxLikJ_FS7uW9EsjiSRUSHSdspcn4wZc6/?ref=159")
	
	def openURL_ASSETS(self, context):
		bpy.ops.wm.url_open(url="https://blendermarket.com/products/simply-asset-packs/?ref=159")
	
	def openURL_MATS(self, context):
		bpy.ops.wm.url_open(url="https://blendermarket.com/products/extreme-pbr-addon-for-blender-279-2/?ref=159")
				
	def openURL_HDRI_Maker(self, context):
		bpy.ops.wm.url_open(url="https://blendermarket.com/products/hdri-maker/?ref=159")
			
	def openURL_RIGS(self, context):
		bpy.ops.wm.url_open(url="https://blendermarket.com/products/auto-rig-pro/?ref=159")
					
	def openURL_VOXELHEAT(self, context):
		bpy.ops.wm.url_open(url="https://blendermarket.com/products/voxel-heat-diffuse-skinning/?ref=159")
					
	def openURL_CHIPP(self, context):
		bpy.ops.wm.url_open(url="https://blendermarket.com/creators/altuit/?ref=159")
		
	def openURL_KCYCLES(self, context):
		bpy.ops.wm.url_open(url="https://blendermarket.com/products/k-cycles/?ref=159")
		
	def openURL_HUMANEGEN(self, context):
		bpy.ops.wm.url_open(url="https://blendermarket.com/products/humgen3d/?ref=159")
		
	def openURL_CLOTHSMOTION(self, context):
		bpy.ops.wm.url_open(url="https://blendermarket.com/creators/xane-graphics/?ref=159")
		
	def openURL_SIMPLYADDONS(self, context):
		bpy.ops.wm.url_open(url="https://blendermarket.com/creators/vjaceslavt/?ref=159")
		
	def openURL_DOC(self, context):
		bpy.ops.wm.url_open(url="https://docs.google.com/document/d/1xYTW1D-_UBNLJrqQ3899ahOQJ_zCjS-v4XWOWQK7mQ0/edit?usp=sharing")
			
	def openURL_CUSTOMERSUPPORT(self, context):
		bpy.ops.wm.url_open(url="https://form.jotform.com/vjaceslavt/simply-problem")
	
	def execute(self, context):
		if self.mode == "VIDEOS":
			self.openURL(context)
		elif self.mode == "CLOTH":
			self.openURL_ASSETS(context)

		elif self.mode == "MATS":
			self.openURL_MATS(context)
                        
		elif self.mode == "HDRI":
			self.openURL_HDRI_Maker(context)

		elif self.mode == "RIGS":
			self.openURL_RIGS(context)
                        
		elif self.mode == "VOXELHEAT":
			self.openURL_VOXELHEAT(context)

		elif self.mode == "CHIPP":
			self.openURL_CHIPP(context)

		elif self.mode == "KCYCLES":
			self.openURL_KCYCLES(context)

		elif self.mode == "HUMANGEN":
			self.openURL_HUMANEGEN(context)

		elif self.mode == "CLOTHSMOTION":
			self.openURL_CLOTHSMOTION(context)

		elif self.mode == "SIMPLYADDONS":
			self.openURL_SIMPLYADDONS(context)

		elif self.mode == "DOCS":
			self.openURL_DOC(context)

		elif self.mode == "CUSTOMERSUPPORT":
			self.openURL_CUSTOMERSUPPORT(context)
		return {'FINISHED'}

class ResetParametersToStandard(bpy.types.Operator):
    bl_idname = "object.sm_reset_material_parameters_to_default"
    bl_label = "Setup Reset"
    bl_description = "Reset all Parameter"

    material = any
    fuzzModifier = any

    def resetAllShaderNodeParameters(self, context):
        self.material.node_tree.nodes["SC_Base_Texture_Scale"].outputs[0].default_value = 5
        self.material.node_tree.nodes["SC_Base_Color_Intensity"].outputs[0].default_value = 1

        self.material.node_tree.nodes["SC_Micro_Texture_Scale"].outputs[0].default_value = 1
        self.material.node_tree.nodes["SC_Micro_Texture_Scale"].outputs[0].default_value = 1
        self.material.node_tree.nodes["SC_Overlay_Texture_Scale"].outputs[0].default_value = 2

        self.material.node_tree.nodes["SM_Overlay_Invert"].mute = True
        self.material.node_tree.nodes["SCM_Overlay_Mask"].mute = True

        self.material.node_tree.nodes["SC_Mix_Custom_Base_Color"].mute = True

        self.material.node_tree.nodes["SC_Mapping_Base"].inputs[1].default_value[0] = 0
        self.material.node_tree.nodes["SC_Mapping_Base"].inputs[1].default_value[1] = 0

        self.material.node_tree.nodes["SC Mapping_Micro"].inputs[1].default_value[0] = 0
        self.material.node_tree.nodes["SC Mapping_Micro"].inputs[1].default_value[1] = 0

        self.material.node_tree.nodes["SC Mapping_Overlay"].inputs[1].default_value[0] = 0
        self.material.node_tree.nodes["SC Mapping_Overlay"].inputs[1].default_value[1] = 0

        self.material.node_tree.nodes["SC_Mapping_Base"].node_tree.nodes["SC_Base_Vector_Scale_Add"].inputs[1].default_value[0] = 0
        self.material.node_tree.nodes["SC_Mapping_Base"].node_tree.nodes["SC_Base_Vector_Scale_Add"].inputs[1].default_value[1] = 0
        self.material.node_tree.nodes["SC_Mapping_Base"].inputs[1].default_value[0] = 0
        self.material.node_tree.nodes["SC_Mapping_Base"].node_tree.nodes["SM_Rotate"].inputs[3].default_value = 0
        self.material.node_tree.nodes["SC_Mapping_Base"].inputs[1].default_value[0] = 0

        self.material.node_tree.nodes["SC Mapping_Micro"].node_tree.nodes["SC_Micro_Vector_Scale_Add"].inputs[1].default_value[0] = 0
        self.material.node_tree.nodes["SC Mapping_Micro"].node_tree.nodes["SC_Micro_Vector_Scale_Add"].inputs[1].default_value[1] = 0
        self.material.node_tree.nodes["SC Mapping_Micro"].node_tree.nodes["SM_Rotate"].inputs[3].default_value = 0
        self.material.node_tree.nodes["SC Mapping_Micro"].inputs[1].default_value[0] = 0
        self.material.node_tree.nodes["SC Mapping_Micro"].inputs[1].default_value[1] = 0

        self.material.node_tree.nodes["SC Mapping_Overlay"].node_tree.nodes["SC_Overlay_Vector_Scale_Add"].inputs[1].default_value[0] = 0
        self.material.node_tree.nodes["SC Mapping_Overlay"].node_tree.nodes["SC_Overlay_Vector_Scale_Add"].inputs[1].default_value[1] = 0
        self.material.node_tree.nodes["SC Mapping_Overlay"].node_tree.nodes["SM_Rotate"].inputs[3].default_value = 0
        self.material.node_tree.nodes["SC Mapping_Overlay"].inputs[1].default_value[0] = 0
        self.material.node_tree.nodes["SC Mapping_Overlay"].inputs[1].default_value[1] = 0

        # self.material.node_tree.node_groups["SC_Mapping_Base"].nodes["SM_Rotate"].inputs[3].default_value = 1.60745

        self.material.node_tree.nodes["SM_Base_Color_Invert"].mute = True
        self.material.node_tree.nodes["BaseTexture_ColorRamp"].mute = False

        self.material.node_tree.nodes["SM_Mix_Micro_to_Base_Intensity"].outputs[0].default_value = 0
        self.material.node_tree.nodes["SM_Mix_Overlay_to_Base_Intensity"].outputs[0].default_value = 0
        self.material.node_tree.nodes["SCM_Overlay_Mask"].mute = True

        self.material.node_tree.nodes["SM_Roughness_Blend"].blend_type = 'MULTIPLY'

        self.material.node_tree.nodes["SM_Roughness_MIX"].outputs[0].default_value = 1
        self.material.node_tree.nodes["SM_Roughness"].outputs[0].default_value = 0.5
        self.material.node_tree.nodes["SM_Roughness_Invert"].mute = True

        self.material.node_tree.nodes["SM_Principled_BSDF"].inputs[6].default_value = 0
        self.material.node_tree.nodes["SM_Principled_BSDF"].inputs[7].default_value = 0.25

        self.material.node_tree.nodes["SM_Principled_BSDF"].inputs[8].default_value = (0.5,0.5,0.5)
        self.material.node_tree.nodes["SM_Principled_BSDF"].inputs[14].default_value = 0
        self.material.node_tree.nodes["SM_Principled_BSDF"].inputs[15].default_value = 0

        self.material.node_tree.nodes["SCM_Custom_Normal_Intensity"].outputs[0].default_value = 1
        self.material.node_tree.nodes["SCM_Custom_Normal_Micro_Mix_Intensity"].outputs[0].default_value = 1
        self.material.node_tree.nodes["SM_Micro_Invert"].mute = False

        self.material.node_tree.nodes["SC_Global_Normal_Intensity"].outputs[0].default_value = 1
        self.material.node_tree.nodes["SC_Base_Normal_Intensity"].outputs[0].default_value = 1
        self.material.node_tree.nodes["SCM_Mask_Base_to_Micro"].mute = True

        self.material.node_tree.nodes["SM_Normal_Global"].invert = False
        self.material.node_tree.nodes["SM_Base_Normal_Invert"].mute = True
        self.material.node_tree.nodes["SCM_INVERT_Mask_Base_to_Micro"].mute = True

        self.material.node_tree.nodes["SC Micro Texture Intensity"].outputs[0].default_value = 10
        self.material.node_tree.nodes["SC_Overlay_Normal_Intensity"].outputs[0].default_value = 10

        self.material.node_tree.nodes["SM_MixNormal_BaseToMicro"].blend_type = 'MULTIPLY'
        self.material.node_tree.nodes["SCM_Overlay_Normal_Mode"].blend_type = 'LINEAR_LIGHT'

        self.material.node_tree.nodes["SC_Base_Alpha"].outputs[0].default_value = 0
        self.material.node_tree.nodes["SC_Base_Alpha_Mix"].outputs[0].default_value = 1
        self.material.node_tree.nodes["SC_Alpha_Invert"].mute = True

        self.material.node_tree.nodes["SC_SubSurface_Intensity"].outputs[0].default_value = 0
        self.material.node_tree.nodes["SM Subsurface Blend Mode"].blend_type = 'SCREEN'
  
        self.material.node_tree.nodes["SM_Sheen_Fresnel_Value"].outputs[0].default_value = 1
        self.material.node_tree.nodes["SM_Sheen_Fresnel_Tint"].outputs[0].default_value = 1

        self.material.node_tree.nodes["SM_Principled_BSDF"].inputs[17].default_value = 0
        self.material.node_tree.nodes["SM_Principled_BSDF"].inputs[18].default_value = 0


        self.material.node_tree.nodes["SM_Emission_Intensity"].outputs[0].default_value = 0
        self.material.node_tree.nodes["SM_Emission_Contrast"].outputs[0].default_value = 0
        self.material.node_tree.nodes["SCM_Emission_Invert"].mute = False

        self.material.node_tree.nodes["SM_Dirt_Node"].node_tree.nodes["Dirt_Intensity"].outputs[0].default_value = 0
        self.material.node_tree.nodes["SM_Dirt_Node"].node_tree.nodes["Dirt_Noise"].inputs[2].default_value = 5
        self.material.node_tree.nodes["SM_Dirt_Node"].node_tree.nodes["Dirt_Noise"].inputs[3].default_value = 5
        self.material.node_tree.nodes["SM_Dirt_Node"].node_tree.nodes["Dirt_Noise"].inputs[4].default_value = 0.5
        self.material.node_tree.nodes["SM_Dirt_Node"].node_tree.nodes["Dirt_Noise"].inputs[5].default_value = 0.1

        self.material.node_tree.nodes["SM_Tearing_Node"].node_tree.nodes["Tear_Intensity_Global"].outputs[0].default_value = 0
        self.material.node_tree.nodes["SM_Tearing_Node"].node_tree.nodes["SCM_Tearing_Mix_Texture_To_Noise"].mute = True
        self.material.node_tree.nodes["SM_Tearing_Node"].node_tree.nodes["SCM_Tearing_Mix_Noise_Paint"].outputs[0].default_value = 0
        self.material.node_tree.nodes["SCM_Tear_Noise"].inputs[2].default_value = 5
        self.material.node_tree.nodes["SM_Tearing_Node"].node_tree.nodes["Fiber_Normal_Intensity"].outputs[0].default_value = 1
        self.material.node_tree.nodes["SCM_Tear_Noise"].inputs[4].default_value = 0.5
        self.material.node_tree.nodes["SM_Tearing_Node"].node_tree.nodes["Tear_Fiber_Intensity"].outputs[0].default_value = 1
        self.material.node_tree.nodes["SM_Tearing_Node"].node_tree.nodes["Tear_Fiber_Scale"].outputs[0].default_value = 50
        self.material.node_tree.nodes["SM_Tearing_Node"].node_tree.nodes["Tear_Fringle_intensity"].outputs[0].default_value = 0
        self.material.node_tree.nodes["SM_Tearing_Node"].node_tree.nodes["Tear_Fiber_Distort"].outputs[0].default_value = 5
        self.material.node_tree.nodes["SM_Tearing_Node"].node_tree.nodes["RGB_TEAR_COLOR"].outputs[0].default_value = (0.05, 0.04, 0.03, 1)

        self.material.node_tree.nodes["SM_Glitter_Intensity"].outputs[0].default_value = 0
        self.material.node_tree.nodes["SCM_Glitter_Normal_Mix"].mute = True
        self.material.node_tree.nodes["SM_Glitter_Scale"].outputs[0].default_value = 50
        self.material.node_tree.nodes["SM_Glitter_Voronoi"].inputs[5].default_value = 1

        self.material.node_tree.nodes["SM_Glitter_Smooth"].outputs[0].default_value = 0
        self.material.node_tree.nodes["SM_Glitter_Roughness_Mix"].outputs[0].default_value = 0
        self.material.node_tree.nodes["SM_Mask_Base_for_Glitter"].mute = True
        self.material.node_tree.nodes["SM_Mask_Base_for_Glitter_Invert"].mute = True

        self.material.node_tree.nodes["SM_Node_Group_Wearing"].node_tree.nodes["SCM_Wear_Mix"].mute = True
        self.material.node_tree.nodes["SM_Wearing_Cavity_Invert"].mute = True
        self.material.node_tree.nodes["SM_Node_Group_Wearing"].node_tree.nodes["SCM_Wear_Intensity"].outputs[0].default_value = 1
        self.material.node_tree.nodes["SM_Wearing_Normal_Intensity"].outputs[0].default_value = 1
        self.material.node_tree.nodes["SC_Wear_Noise_Texture_Scale"].outputs[0].default_value = 10


        self.material.node_tree.nodes["SCM_Mix_Velvet"].mute = True
        self.material.node_tree.nodes["SCM_Velvet_Balance"].outputs[0].default_value = 0.7

        self.material.node_tree.nodes["SCM_Mix_Dispersion"].mute = True
        self.material.node_tree.nodes["SCM_Dispersion_Balance"].outputs[0].default_value = 1
        self.material.node_tree.nodes["SCM_Dispersion"].outputs[0].default_value = 1
        self.material.node_tree.nodes["SCM_Dispersion_Roughness"].outputs[0].default_value = 0
        self.material.node_tree.nodes["SCM_Dispersion_IOR"].outputs[0].default_value = 1.45
        self.material.node_tree.nodes["SC_Global_Normal_Intensity"].outputs[0].default_value = 1

        self.material.node_tree.nodes["SM_Backface_Shader"].mute = True
        self.material.node_tree.nodes["SM_Backface_Color_Mix_Intensity"].outputs[0].default_value = 1
        self.material.node_tree.nodes["SM_Backface_Color_Brightness"].outputs[0].default_value = 1
        self.material.node_tree.nodes["SM_Backface_Color_Saturation"].outputs[0].default_value = 1
        self.material.node_tree.nodes["SM_Backface_Color_Hue"].outputs[0].default_value = 0.5
        self.material.node_tree.nodes["SM_Backface_Color_Mix"].blend_type = 'MIX'
        self.material.node_tree.nodes["SM_Backface_Color"].outputs[0].default_value = (1, 1, 1, 1)

        self.material.node_tree.nodes["SCM_Node_Blur_Base"].node_tree.nodes["SCM_Node_Blur_Intensity"].outputs[0].default_value = 0.0


        self.material.node_tree.nodes["BaseTexture_ColorRamp"].mute = True
        self.material.node_tree.nodes["BaseTexture_HueSatValue"].mute = True

        self.material.node_tree.nodes["BaseTexture_HueSatValue"].inputs[0].default_value = 0.5
        self.material.node_tree.nodes["BaseTexture_HueSatValue"].inputs[1].default_value = 1
        self.material.node_tree.nodes["BaseTexture_HueSatValue"].inputs[2].default_value = 1

        self.material.node_tree.nodes["BaseTexture_ColorRamp"].color_ramp.elements[0].color = (0, 0, 0, 1)
        self.material.node_tree.nodes["SM_Backface_Color"].outputs[0].default_value = (1, 1, 1, 1)

        self.material.node_tree.nodes["SC_Base_Color_CustomTextureMix_Intensity"].outputs[0].default_value = 0
        self.material.node_tree.nodes["SC_Base_Color_CustomTextureMix_Blend"].mute = True


        if "SM_Fuzz_Geo_Nodes" in bpy.data.node_groups:
            # self.material.show_render = False
            # self.material.show_viewport = False
            # self.material.show_in_editmode = False
            bpy.data.node_groups["SM_Fuzz_Geo_Nodes"].nodes["SM_Fuzzle_Render_Viewport_Density"].inputs[3].default_value = 100
            bpy.data.node_groups["SM_Fuzz_Geo_Nodes"].nodes["SM_Fuzzle_Render_Viewport_Density"].inputs[2].default_value = 200
            bpy.data.node_groups["SM_Fuzz_Geo_Nodes"].nodes["SM_Fuzz_Node_Global_Density"].outputs[0].default_value = 500

            bpy.data.node_groups["SM_Fuzz_Geo_Nodes"].nodes["SM_Fuzz_Node_Normal_Density"].outputs[0].default_value = 1
            bpy.data.node_groups["SM_Fuzz_Geo_Nodes"].nodes["SM_Fuzz_Node_Distorted_Density"].outputs[0].default_value = 1
            bpy.data.node_groups["SM_Fuzz_Geo_Nodes"].nodes["SM_Fuzz_Node_Curly_Density"].outputs[0].default_value = 1

            bpy.data.node_groups["SM_Fuzz_Geo_Nodes"].nodes["SM_Fuzz_Node_Normal"].inputs[3].default_value = 1
            bpy.data.node_groups["SM_Fuzz_Geo_Nodes"].nodes["SM_Fuzz_Node_Distorted"].inputs[3].default_value = 2
            bpy.data.node_groups["SM_Fuzz_Geo_Nodes"].nodes["SM_Fuzz_Node_Curly"].inputs[3].default_value = 3

            bpy.data.node_groups["SM_Fuzz_Geo_Nodes"].nodes["SM_Fuzz_Size_Global"].outputs[0].default_value = 1
            bpy.data.node_groups["SM_Fuzz_Geo_Nodes"].nodes["SM_Fuzz_Size_Min"].outputs[0].default_value = 0.1
            bpy.data.node_groups["SM_Fuzz_Geo_Nodes"].nodes["SM_Fuzz_Size_Max"].outputs[0].default_value = 1

            bpy.data.node_groups["SM_Fuzz_Geo_Nodes"].nodes["SM_Fuzz_Node_Normal_Scale"].outputs[0].default_value = 1
            bpy.data.node_groups["SM_Fuzz_Geo_Nodes"].nodes["SM_Fuzz_Node_Distorted_Scale"].outputs[0].default_value = 1
            bpy.data.node_groups["SM_Fuzz_Geo_Nodes"].nodes["SM_Fuzz_Node_Curly_Scale"].outputs[0].default_value = 1
            
            bpy.data.node_groups["SM_Fuzz_Geo_Nodes"].nodes["SM_Fuzz_Fill_Caps"].boolean = True
            bpy.data.node_groups["SM_Fuzz_Geo_Nodes"].nodes["SM_Fuzz_Root_Thickness"].outputs[0].default_value = 12
            bpy.data.node_groups["SM_Fuzz_Geo_Nodes"].nodes["SM_Fuzz_Tip_Thickness"].outputs[0].default_value = 1

    def resetAllUIParameters(self, context):
        bpy.context.object.scm_Base_Intensity = 1
        bpy.context.object.scm_Micro_Intensity = 1
        bpy.context.object.scm_Overlay_Intensity = 1
        bpy.context.object.scm_UI_Additional_Image = False
        bpy.context.object.scm_UI_normal_custom_active = False


    def getCurrentMaterial(self, context):
        for i, mat in enumerate(context.active_object.material_slots):
            if "SimplyMaterialInfo" in mat.material.node_tree.nodes:
                self.simplyMaterialSlot = i
                self.material = mat.material

    def getCurrentFuzzModifier(self, context):
        for i, modifier in enumerate(context.active_object.modifiers):
            if "SimplyGeoNode" in modifier.name:
                self.fuzzModifier = modifier

    def execute(self, context):
        self.getCurrentMaterial(context)
        self.getCurrentFuzzModifier(context)
        self.resetAllShaderNodeParameters(context)
        self.resetAllUIParameters(context)
        return {'FINISHED'}

classes = (
    SwitchSelectMainTextures,
    RefreshAvailableTextures,
    CyclesExperimentalAdaptiveSubdivisionSetup,
    SetUpMaterialForActiveObject,
    CleanUpSimplyImagesFromBlendFile,
    SetUpAndRenderCavityImageTextureForMaterial,
    SetupPreviewForCavityBaking,
    BakeOutputTextures,
    SwitchToPaintModeAndActivateCorrectImageTexture,
    SwitchToPaintModeAndActivateCorrectTearingImageTexture,
    SetupSimplyFuzzleGeometryNodes,
    ResetParametersToStandard,
    SwitchToPaintModeAndActivateCustomBaseColorImageTexture,
    SetupModifierForDisplacementUsage,
    OpenURL
)


def register():
    # register all classes
    for cls in classes:
        bpy.utils.register_class(cls)


def unregister():
    # unregister all classes
    for cls in classes:
        bpy.utils.unregister_class(cls)