# Simply Cloth Material by Vjaceslav Tissen
# vjaceslavt@gmail.com#
# In Cooperation with Daniel Meier
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTIBILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


import bpy
import os

from . import ui_panel

from bpy.props import EnumProperty, IntProperty, FloatProperty, StringProperty, BoolProperty
            
def register():
   bpy.types.Object.scm_imageCount = IntProperty(default=0, description="Image Count of all images from Simply Material")

   bpy.types.Object.scm_imageBaseIndex = IntProperty(default=0, description="Image Index of Base Texture")
   bpy.types.Object.scm_imageMicroIndex = IntProperty(default=0,  description="Image Index of Micro Texture")
   bpy.types.Object.scm_imageOverlayIndex = IntProperty(default=0,  description="Image Index of Overlay Texture")

   bpy.types.Object.scm_materialSlot = IntProperty(default=0,  description="Simply Material Slot")
   bpy.types.Object.scm_materialName = StringProperty(default="",  description="Material Name of active object")


   bpy.types.Object.scm_Base_Intensity = FloatProperty(default=1, description="Image Intensity", min=0.0, max=1.0, update=ui_panel.updateBaseIntensity)
   bpy.types.Object.scm_Micro_Intensity = FloatProperty(default=1, description="Image Intensity", min=0.0, max=1.0, update=ui_panel.updateMicroIntensity)
   bpy.types.Object.scm_Overlay_Intensity = FloatProperty(default=1, description="Image Intensity", min=0.0, max=1.0, update=ui_panel.updateOverlayIntensity)
   bpy.types.Object.scm_Additional_Intensity = FloatProperty(default=1, description="Image Intensity", min=0.0, max=1.0, update=ui_panel.updateAdditionalIntensity)


   bpy.types.Object.scm_UI_Normals = BoolProperty(default=False, description="Set UI Normal on/ off")
   bpy.types.Object.scm_UI_Glossy = BoolProperty(default=False, description="Set UI Roughness/ Metallic on/ off")
   bpy.types.Object.scm_UI_ExtraSurfaceShading = BoolProperty(default=False, description="Set UI Subsurface/ Fresnel and Emission on/ off")

   bpy.types.Object.scm_UI_BaseColor = BoolProperty(default=False, description="Set UI Base Color / Ramp on/ off")
   bpy.types.Object.scm_UI_Alpha = BoolProperty(default=False, description="Set UI Alpha Color/ Ramp on/ off")
   bpy.types.Object.scm_UI_MicroColor = BoolProperty(default=False, description="Set Micro Overlay Color/ Ramp on/ off")
   bpy.types.Object.scm_UI_Overlay = BoolProperty(default=False, description="Set Overlay Roughness/ Ramp on/ off")


   bpy.types.Object.scm_UI_COLORS = BoolProperty(default=False, description="Set Colors on/ off")
   bpy.types.Object.scm_UI_TRANSFORM = BoolProperty(default=False, description="Set Transform and Rotation on/ off")

   bpy.types.Object.scm_UI_ExportForUnrealEngine = BoolProperty(default=False, description="Export for UE Engine")
   bpy.types.Scene.scm_UI_PreviewForCavityBake = BoolProperty(default= False, description="preview UI for Cavity Bake")


   bpy.types.Object.scm_DEV_Workflow = BoolProperty(default=True, description="Workflow")

   bpy.types.Object.scm_UI_Images = BoolProperty(default=False, description="Base, Micro, Overlay Images UI open/ close")
   bpy.types.Object.scm_UI_Additional_Image = BoolProperty(default=False, description="Open Additional Effects UI open/ close",update=ui_panel.switchAdditionalImageTexture)
   bpy.types.Object.scm_UI_Roughness_Image = BoolProperty(default=False, description="Open Additional Roughness Image open/ close",update=ui_panel.switchAdditionalImageTexture)

   bpy.types.Object.scm_UI_Additional_Effects = BoolProperty(default=False, description="Open Additional Effects UI open/ close")
   
   # bpy.types.Object.scm_UI_Dirt = BoolProperty(default=False, description="Set Dirt on/ off", update=switch_dirt_bool)
   # bpy.types.Object.scm_UI_Blur = BoolProperty(name="SCM Blur", description="Set Blur on/ off",default=False, update=ui_panel.update_blur_nodes)
   bpy.types.Object.scm_UI_Subsurface_Sheen= BoolProperty(default=False, description="Set Sub Surface on/ off")
   bpy.types.Object.scm_UI_Emission= BoolProperty(default=False, description="Set Emissive on/ off")
   bpy.types.Object.scm_UI_Dirt = BoolProperty(default=False, description="Set Dirt on/ off")
   bpy.types.Object.scm_UI_Wearing = BoolProperty(default=False, description="Set Wearing on/ off")
   bpy.types.Object.scm_UI_Velvet = BoolProperty(default=False, description="velvet on/ off", name="velvet")
   bpy.types.Object.scm_UI_Dispersion = BoolProperty(default=False, description="dispersion on/ off", name="dispersion")
   bpy.types.Object.scm_UI_Glitter = BoolProperty(default=False, description="glitter on/ off", name="glitter")
   bpy.types.Object.scm_UI_Fuzzle = BoolProperty(default=False, description="Fuzzle on/ off", name="fuzzle")
   bpy.types.Object.scm_UI_Backface = BoolProperty(default=False, description="Backface on/ off", name="backface")

   bpy.types.Object.scm_UI_Tear = BoolProperty(default=False, description="Set Tear on/ off")
   bpy.types.Object.scm_UI_Tear_Paint = BoolProperty(default=False, description="Set Tear Paint on/ off")
   bpy.types.Object.scm_UI_Tear_Procedural = BoolProperty(default=False, description="Set Tear Noise on/ off")

   bpy.types.Object.scm_UI_Bake = BoolProperty(default=False, description="Bake Settings on/ off")

   bpy.types.Object.scm_UI_Bake_Diffuse = BoolProperty(default=False, description="Bake Settings on/ off", name="basecolor")
   bpy.types.Object.scm_UI_Bake_Normal = BoolProperty(default=False, description="Bake Settings on/ off", name="normal_opgl")
   bpy.types.Object.scm_UI_Bake_Normal_UE = BoolProperty(default=False, description="Bake Settings on/ off", name="normal_ue")
   bpy.types.Object.scm_UI_Bake_Roughness = BoolProperty(default=False, description="Bake Settings on/ off", name="roughness")
   bpy.types.Object.scm_UI_Bake_AO = BoolProperty(default=False, description="Bake Settings on/ off", name="ambient_occlusion")
   bpy.types.Object.scm_UI_Bake_Emissive = BoolProperty(default=False, description="Bake Settings on/ off", name="emissive")
   bpy.types.Object.scm_UI_Bake_Alpha = BoolProperty(default=False, description="Bake Settings on/ off", name="alpha")
   bpy.types.Object.scm_UI_Bake_Cavity = BoolProperty(default=False, description="Bake Settings on/ off", name="alpha")
   bpy.types.Object.scm_UI_Bake_UV = BoolProperty(default=False, description="UV Bake on/ off", name="uv")
   
   bpy.types.Object.scm_geoNodes_fuzzle_modifier_name = StringProperty(default="", description="image output name")

   bpy.types.Object.scm_image_output_name = StringProperty(default="", description="image output name")
   bpy.types.Object.scm_image_custom_base_color_name = StringProperty(default="BC_custom_base_color_texture", description="custom base color image texture name")


   bpy.types.Object.scm_image_output_width = IntProperty(default=1024, description="image resolution width")
   bpy.types.Object.scm_image_output_height = IntProperty(default=1024, description="image resolution height")
   
   bpy.types.Object.scm_UI_normal_custom_active = BoolProperty(default=False, description="Enable custom Normal Texture on/ off", update=ui_panel.switchCustomNormal)
   bpy.types.Object.scm_UI_roughness_custom_active = BoolProperty(default=False, description="Enable custom Roughness Texture on/ off", update=ui_panel.switchCustomRoughness)
   
   bpy.types.Object.scm_Fuzzle_Modifier = BoolProperty(default=False, description="Fuzzle Modifier on/ off")

   bpy.types.Scene.scm_UI_Support = BoolProperty(default=False, description="Support Page on/ off", name="support")

   ui_panel.registerIcon()

def unregister():
   ui_panel.unregisterIcon()
   pass