# Simply Cloth Material by Vjaceslav Tissen
# vjaceslavt@gmail.com
# In Cooperation with Daniel Meier

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTIBILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
from pathlib import Path
from cgitb import enable
import bpy
import os

class SimplyMaterialBasePanel(bpy.types.Panel):
    bl_idname = "SIMPLYMATERIAL_PT_Panel"
    bl_label = "Simply Material | v.1.3"
    bl_category = "Simply Addons"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'

    simplyMat = any

    # scm_materialSlot = getCurrentMaterialSlotFromActiveObject
    # print(scm_materialSlot)

    def draw(self, context):
        layout = self.layout
        scene = context.scene
        active_object = context.active_object
        simplyMaterial = None

        simplyFuzzleModifier = None
        simplyFuzzleGeoNodes = None

        if context.mode == "OBJECT":
            box = layout.box()
            row = box.row()
            simplyStartIcon = custom_icons["simplyStartIcon"].icon_id
            row.label(text="Simply Start", icon_value=simplyStartIcon)
            row.label(text="", icon="BLANK1")
            op = row.operator("outliner.orphans_purge", text="Clean Unused Images", icon="TRASH")
            row = box.row()
            row.scale_y = 2.0
            row.active_default = True
            row.label(text="", icon="PRESET_NEW")
            row.operator("object.sm_setup_material_for_active_object", text="Create Simply Material", icon="PLUS")
            row.operator("object.sm_reset_material_parameters_to_default", text="Reset all Changes", icon="RECOVER_LAST")


            op.do_recursive=True
            op.do_local_ids=True
            op.do_linked_ids=True


        if context.active_object:
            if active_object.scm_materialSlot is not None:

                try:
                    simplyMaterial =bpy.data.materials[active_object.material_slots[context.active_object.material_slots[context.active_object.scm_materialSlot].name].name]
                    self.simplyMat = simplyMaterial
                except:
                    pass

                if active_object is not None:
                    if context.mode == "PAINT_TEXTURE":
                        if simplyMaterial.node_tree.nodes["SC_Base_Color_CustomTextureMix_Texture"].image is not None:
                            box = layout.box()
                            row = box.row()
                            row.label(text="Custom Base Color Texture", icon="IMAGE")
                            row.label(text="", icon="BLANK1")
                            row.prop(bpy.data.images[context.active_object.scm_image_custom_base_color_name], "source", text="")
                            row = box.row()
                            if bpy.data.images[context.active_object.scm_image_custom_base_color_name].source == "FILE":
                                # row = box.row()
                                row.label(text="", icon="BLANK1")
                                row.prop(simplyMaterial.node_tree.nodes["SC_Base_Color_CustomTextureMix_Texture"].image, "filepath", text="", icon="IMAGE_BACKGROUND")
                                row = box.row()
                            if bpy.context.space_data.shading.type == 'MATERIAL' or bpy.context.space_data.shading.type == 'RENDERED':
                                # row = box.row()
                                row.label(text="", icon="IMAGE_DATA")
                                row.prop(simplyMaterial.node_tree.nodes["SC_Base_Color_CustomTextureMix_Intensity"].outputs[0], "default_value", text="Texture Intensity", invert_checkbox=True, icon="MOD_MASK")
                                row.prop(simplyMaterial.node_tree.nodes["SC_Base_Color_CustomTextureMix_Blend"], "blend_type", text="", icon="NODE_TEXTURE")
                                row = box.row()
                        box = layout.box()
                        row = box.row()
                        row.label(text="Overlay Texture Paint", icon="BRUSHES_ALL")

                        if context.active_object.material_slots[context.active_object.scm_materialSlot].material.node_tree is not None:
                            node_tree = context.active_object.material_slots[context.active_object.scm_materialSlot].material.node_tree

                        if node_tree.nodes['SC_Overlay_Texture'].image is not None:
                            row = box.row()
                            row.label(text="Overlay Texture", icon="UV")
                            row.prop(simplyMaterial.node_tree.nodes["SC_Overlay_Texture"].image, "filepath", text="", icon="IMAGE_BACKGROUND")
                            row = box.row()
                            if simplyMaterial.node_tree.nodes["SCM_Overlay_Mask"].mute == True:
                                row.alert=True
                            
                            if simplyMaterial.node_tree.nodes["SC_Overlay_Texture_Scale"].outputs[0].default_value <0:
                                row.alert =True
                            
                            row.prop(simplyMaterial.node_tree.nodes["SCM_Overlay_Mask"], "mute", text="", icon="HIDE_OFF", invert_checkbox=True)
                            row.prop(simplyMaterial.node_tree.nodes["SC_Overlay_Texture_Scale"].outputs[0], "default_value", text="Scale")
                            row.prop(context.active_object, "scm_Overlay_Intensity", text="Intensity", slider=True)
                            row.prop(simplyMaterial.node_tree.nodes["SM_Overlay_Invert"], "mute", text="", icon="IMAGE_ALPHA", invert_checkbox=True)
                            
                        # TEARING TEXTURE PAINT
                        box = layout.box()
                        row = box.row()
                        row.label(text="Tearing Texture Paint", icon="BRUSHES_ALL")
                        row.prop(simplyMaterial.node_tree.nodes["SM_Tearing_Node"].node_tree.nodes["SCM_Tearing_Mix_Noise_Paint"].outputs[0], "default_value", text="Mix Procedural + Texture Paint")

                        if context.active_object.material_slots[context.active_object.scm_materialSlot].material.node_tree is not None:
                            node_tree = context.active_object.material_slots[context.active_object.scm_materialSlot].material.node_tree
                        if node_tree.nodes['SCM_Tear_Paint_Texture'].image is not None:
                            row = box.row()
                            if simplyMaterial.node_tree.nodes["SCM_Tear_Noise"].mute == True:
                                row.alert=True

                            row.prop(simplyMaterial.node_tree.nodes["SCM_Tear_Noise"], "mute", text="Procedural", icon="HIDE_OFF", invert_checkbox=True)
                            row.prop(simplyMaterial.node_tree.nodes["SM_Tearing_Node"].node_tree.nodes["SCM_Tearing_Noise_Intensity"].outputs[0], "default_value", text="Noise Intensity")
                            row = box.row()                     
                            
                            if simplyMaterial.node_tree.nodes["SM_Tearing_Node"].node_tree.nodes["SCM_Tearing_Mix_Texture_To_Noise"].mute == True:
                                row.alert=True

                            row.prop(simplyMaterial.node_tree.nodes["SM_Tearing_Node"].node_tree.nodes["SCM_Tearing_Mix_Texture_To_Noise"], "mute", text="Texture Paint", icon="HIDE_OFF", invert_checkbox=True)
                            row.prop(simplyMaterial.node_tree.nodes["SM_Tearing_Node"].node_tree.nodes["SCM_Tearing_Paint_Texture_Intensity"].outputs[0], "default_value", text="Texture Intensity")
                        
                        box = layout.box()
                        row = box.row() 
                        row.label(text="Finish")
                        row = box.row() 
                        row.scale_y = 2.0
                        row.active_default = True
                        row.operator("paint.texture_paint_toggle",text="Done")         

                    if context.mode == "OBJECT":
                        simplyMaterial = None
                        if context.active_object.material_slots:
                            if context.active_object.material_slots[context.active_object.scm_materialSlot].material.node_tree is not None:
                                node_tree = context.active_object.material_slots[context.active_object.scm_materialSlot].material.node_tree
                        
                                #Simply Material is detected
                                activeObjectMaterialName = context.active_object.material_slots[context.active_object.scm_materialSlot].name
                                simplyMaterial =bpy.data.materials[active_object.material_slots[context.active_object.material_slots[context.active_object.scm_materialSlot].name].name]
                                self.simplyMat = simplyMaterial

                                # if context.active_object.scm_UI_Images == True:
                                #TEXTURES SETTINGS
                                if "SC_Base_Image" in node_tree.nodes:
                                    box = layout.box()
                                    row = box.row()
                                    row.label(text="Blend Mode", icon="TEXTURE_DATA")
                                    row.label(text="", icon="BLANK1")
                                    row.prop(simplyMaterial, "blend_method", text="")
                                    row.prop(simplyMaterial, "shadow_method", text="")
                                    box = layout.box()
                                    row = box.row()
                                    row.label(text="Simply Textures", icon="NODE_COMPOSITING")
                                    # row = box.row()
                                    row.prop(active_object, "scm_UI_Bake", text="Export Textures", icon="EXPORT")
                                    row = box.row()
                                    row.scale_y = 2.0
                                    row.prop(context.active_object, "scm_UI_Images", text="Base | Micro | Overlay | Additional", icon="RENDERLAYERS")


                                    if context.active_object.scm_UI_Images:
                                        if node_tree.nodes['SC_Base_Image'].image is not None:
                                            box = layout.box()
                                            row = box.row()

                                            if simplyMaterial.node_tree.nodes["SC_Base_Image"].mute == False:
                                                row.prop(simplyMaterial.node_tree.nodes["SC_Base_Image"], "mute", text="", icon="HIDE_OFF", invert_checkbox=True)
                                            if simplyMaterial.node_tree.nodes["SC_Base_Image"].mute == True:
                                                row.alert= True
                                                row.prop(simplyMaterial.node_tree.nodes["SC_Base_Image"], "mute", text="", icon="HIDE_ON")

                                            row.label(text="Base Texture", icon="IMAGE_DATA")
                                            row.prop(simplyMaterial.node_tree.nodes["SC_Base_Image"].image, "filepath", text="", icon="IMAGE_BACKGROUND")
                                            
                                            if simplyMaterial.node_tree.nodes["SC_Base_Texture_Scale"].outputs[0].default_value <0:
                                                row.alert =True
                                                
                                            row = box.row()
                                            
                                            row.label(text="", icon="BLANK1")
                                            row.prop(simplyMaterial.node_tree.nodes["SC_Base_Texture_Scale"].outputs[0], "default_value", text="Scale")
                                            row.prop(context.active_object, "scm_Base_Intensity", text="Intensity", slider=True)
                                            
                                            # WIP
                                            row = box.row()
                                            row.label(text="Experimental - Work in Progress", icon="INFO")
                                            row = box.row()
                                            row.alert = True
                                            row.label(text="", icon="IMAGE")
                                            row.prop(simplyMaterial.node_tree.nodes["SC_Base_Color_CustomTextureMix_Blend"], "mute", text="Use Custom Texture for Blend", icon="HIDE_OFF", invert_checkbox= True)
                                            row.prop(simplyMaterial.node_tree.nodes["SC_Base_Color_CustomTextureMix_Intensity"].outputs[0], "default_value", text="Texture Intensity", invert_checkbox=True, icon="MOD_MASK")
                                            if simplyMaterial.node_tree.nodes["SC_Base_Color_CustomTextureMix_Blend"].mute == False:
                                                if context.active_object.scm_image_custom_base_color_name in bpy.data.images:
                                                    if bpy.data.images[context.active_object.scm_image_custom_base_color_name] == simplyMaterial.node_tree.nodes["SC_Base_Color_CustomTextureMix_Texture"].image:
                                                        op = row.operator("object.scm_switch_custom_basecolor_texture_paint", text="", icon="TRASH")
                                                        op.mode = "REMOVE"
                                                        row = box.row()
                                                        row.label(text="", icon="BLANK1")
                                                        op = row.operator("object.scm_switch_custom_basecolor_texture_paint", text="Save Image First!", icon="DISC")
                                                        op.mode = "SAVEIMAGE"
                                                        row = box.row()
                                                        row.label(text="", icon="IMAGE")
                                                        op = row.operator("object.scm_switch_custom_basecolor_texture_paint", text="Paint Texture", icon="BRUSHES_ALL")
                                                        op.mode = "SWITCH"
                                                else:
                                                    row = box.row()
                                                    row.label(text="", icon="BLANK1")
                                                    op = row.operator("object.scm_switch_custom_basecolor_texture_paint", text="Create Custom Texture Color", icon="PLUS")
                                                    op.mode = "CREATE"
                                                
                                                # row = box.row()
                                                # row.label(text="", icon="BLANK1")
                                                if simplyMaterial.node_tree.nodes["SC_Base_Color_CustomTextureMix_Texture"].image is not None:
                                                    row.prop(bpy.data.images[context.active_object.scm_image_custom_base_color_name], "source", text="")
                                                    if bpy.data.images[context.active_object.scm_image_custom_base_color_name].source == "FILE":
                                                        row = box.row()
                                                        row.label(text="", icon="BLANK1")
                                                        row.prop(simplyMaterial.node_tree.nodes["SC_Base_Color_CustomTextureMix_Texture"].image, "filepath", text="", icon="IMAGE_BACKGROUND")
                                                    if bpy.context.space_data.shading.type == 'MATERIAL' or bpy.context.space_data.shading.type == 'RENDERED':
                                                        row = box.row()
                                                        row.label(text="", icon="IMAGE_DATA")
                                                        row.prop(simplyMaterial.node_tree.nodes["SC_Base_Color_CustomTextureMix_Intensity"].outputs[0], "default_value", text="Texture Intensity", invert_checkbox=True, icon="MOD_MASK")
                                                        row.prop(simplyMaterial.node_tree.nodes["SC_Base_Color_CustomTextureMix_Blend"], "blend_type", text="", icon="NODE_TEXTURE")
                                                        row = box.row()

                                        if node_tree.nodes['SC_Micro_Texture'].image is not None:
                                            box = layout.box()
                                            row = box.row()
                                            if simplyMaterial.node_tree.nodes["SC_Micro_Texture"].mute == False:
                                                row.prop(simplyMaterial.node_tree.nodes["SC_Micro_Texture"], "mute", text="", icon="HIDE_OFF", invert_checkbox=True)
                                            if simplyMaterial.node_tree.nodes["SC_Micro_Texture"].mute == True:
                                                row.alert= True
                                                row.prop(simplyMaterial.node_tree.nodes["SC_Micro_Texture"], "mute", text="", icon="HIDE_ON")

                                            row.label(text="Micro Texture", icon="ALIGN_FLUSH")

                                            if simplyMaterial.node_tree.nodes["SC_Micro_Texture_Scale"].outputs[0].default_value <0:
                                                row.alert =True

                                            row.prop(simplyMaterial.node_tree.nodes["SC_Micro_Texture"].image, "filepath", text="", icon="IMAGE_BACKGROUND")
                                            row = box.row()
                                            row.label(text="", icon="BLANK1")
                                            row.prop(simplyMaterial.node_tree.nodes["SC_Micro_Texture_Scale"].outputs[0], "default_value", text="Scale")
                                            row.prop(context.active_object, "scm_Micro_Intensity", text="Intensity", slider=True)

                                        if node_tree.nodes['SC_Overlay_Texture'].image is not None:
                                            box = layout.box()
                                            row = box.row()

                                            if simplyMaterial.node_tree.nodes["SC_Overlay_Texture"].mute == False:
                                                row.prop(simplyMaterial.node_tree.nodes["SC_Overlay_Texture"], "mute", text="", icon="HIDE_OFF", invert_checkbox=True)

                                            if simplyMaterial.node_tree.nodes["SC_Overlay_Texture"].mute == True:
                                                row.alert= True
                                                row.prop(simplyMaterial.node_tree.nodes["SC_Overlay_Texture"], "mute", text="", icon="HIDE_ON")
                                            
                                            row.label(text="Overlay Texture", icon="UV")

                                            if simplyMaterial.node_tree.nodes["SC_Overlay_Texture_Scale"].outputs[0].default_value <0:
                                                row.alert =True

                                            row.prop(simplyMaterial.node_tree.nodes["SC_Overlay_Texture"].image, "filepath", text="", icon="IMAGE_BACKGROUND")
                                            row = box.row()

                                            row.prop(simplyMaterial.node_tree.nodes["SM_Overlay_Invert"], "mute", text="", icon="IMAGE_ALPHA", invert_checkbox=True)
                                            row.prop(simplyMaterial.node_tree.nodes["SC_Overlay_Texture_Scale"].outputs[0], "default_value", text="Scale")
                                            row.prop(context.active_object, "scm_Overlay_Intensity", text="Intensity", slider=True)
                                            #WIP
                                            # row = box.row()

                                            # row.prop(simplyMaterial.node_tree.nodes["SCM_Overlay_Mask"], "mute", text="", icon="HIDE_OFF", invert_checkbox=True)
                                            # row.operator("object.scm_switch_overlay_texture_paint",text="Paint Overlay Texture Mask", icon="BRUSHES_ALL")   

                                        # CUSTOM IMAGE TEXTURE 
                                        if "SC_Custom_Texture_Image" in simplyMaterial.node_tree.nodes:
                                            box = layout.box()
                                            row = box.row()
                                            # row.prop(simplyMaterial.node_tree.nodes["SC_Mix_Custom_Base"], "mute", text="", icon="HIDE_OFF")
                                            # row.label(text="", icon="BLANK1")
                                            row.prop(context.active_object, "scm_UI_Additional_Image", text="", icon="HIDE_OFF")
                                            row.label(text="Additional Image Texture | Logo")
                                            row.prop(simplyMaterial.node_tree.nodes["SC_Custom_Texture_Image"].image, "filepath", text="", icon="IMAGE_BACKGROUND")
                                            # row = box.row()
                                            if context.active_object.scm_UI_Additional_Image==True:
                                                row = box.row()

                                                row.label(text="", icon="BLANK1")
                                                row.prop(simplyMaterial.node_tree.nodes["SC_Custom_Texture_Scale"].outputs[0], "default_value", text="Scale")
                                                row.prop(context.active_object, "scm_Additional_Intensity", text="Intensity", slider=True)
                                                # row.prop(simplyMaterial.node_tree.nodes["SC_Custom_Texture_Intensity"].outputs[0], "default_value", text="Intensity",slider=True)
                                                row = box.row()
                                                row.label(text="", icon="COLOR")
                                                row.prop(simplyMaterial.node_tree.nodes["SC_Mix_Custom_Base_Color"], "blend_type", text="", icon="NODE_TEXTURE")
                                                row = box.row()

                                                # row.label(text="", icon="COLOR")
                                                # row.prop(simplyMaterial.node_tree.nodes["SC_Mix_Custom_Base_Color_Blend"].inputs[0],"default_value", text=" ", icon="COLOR")
                                                # row.prop(simplyMaterial.node_tree.nodes["SC_Mix_Custom_Base_Color_Blend"], "blend_type", text="", icon="NODE_TEXTURE")
                                                # row = box.row()

                                                # CUSTOM IMAGE NORMAL
                                                # row.label(text="", icon="IMAGE_PLANE")
                                                # row.label(text="Normal Blend Mode")
                                                # row.prop(simplyMaterial.node_tree.nodes["SM_MixNormal_CustomImage"], "blend_type", text="", icon="NODE_TEXTURE")
                                                # row = box.row()
                                                # row = box.row()
                                                row.prop(simplyMaterial.node_tree.nodes["SM_Additional_Image_Roughness_Invert"], "mute", text="", icon="IMAGE_ALPHA")
                                                row.prop(simplyMaterial.node_tree.nodes["SM_Additional_Image_Roughness_Intensity"].outputs[0], "default_value", text="Roughness Intensity")
                                                row.prop(simplyMaterial.node_tree.nodes["SM_Additional_Image_Roughness_Blend"], "blend_type", text="", icon="NODE_TEXTURE")
                                                row = box.row()
                                                row.label(text="Transform", icon="IMAGE_BACKGROUND")
                                                # row = box.row()
                                                row.label(text="", icon="BLANK1")
                                                row.prop(simplyMaterial.node_tree.nodes["SC Mapping_Custom"].inputs[1],"default_value", text="X", icon="MOD_DATA_TRANSFER", index=0)
                                                row.prop(simplyMaterial.node_tree.nodes["SC Mapping_Custom"].inputs[1],"default_value", text="Y", icon="MOD_DATA_TRANSFER", index=1)
                                                row.prop(simplyMaterial.node_tree.nodes["SM_Rotate"].inputs[3],"default_value", text="°", icon="DRIVER_ROTATIONAL_DIFFERENCE")

                                                    # row = box.row()
                                    # EXPORT TEXTURES - TEXTURE BAKING
                                    if context.active_object.scm_UI_Bake:
                                        box = layout.box()
                                        row = box.row(align=True)
                                        row.label(text="Output", icon="OUTPUT")
                                        row = box.row()

                                        row.label(text="", icon="BLANK1")
                                        row.active_default = True
                                        row.prop(bpy.data.scenes['Scene'].render, "filepath", text="Export", icon="IMAGE_BACKGROUND")
                                        row = box.row()

                                        row.label(text="", icon="BLANK1")
                                        row.active_default = False
                                        row.prop(bpy.data.scenes['Scene'].render.image_settings, "file_format", text="File Format")
                                        row = box.row()

                                        row.label(text="", icon="BLANK1")
                                        row.prop(context.scene.render.image_settings, "color_depth")
                                        row = box.row()

                                        row.label(text="", icon="BLANK1")
                                        row.prop(context.scene.render.image_settings, "compression", text="File Compression")
                                        row = box.row()

                                        row.label(text="Texture Resolution", icon="SETTINGS")
                                        row.label(text="", icon="BLANK1")
                                        row.prop(context.active_object, "scm_image_output_width", text="px Width")
                                        row.prop(context.active_object, "scm_image_output_height", text="px Height")
                                        # BAKE SECTION
                                        row = box.row()
                                       
                                        row.label(text="Textures for Bake", icon="RENDER_RESULT")
                                        row = box.row()
                                        row.label(text="", icon="BLANK1")
                                        row.prop(context.active_object, "scm_UI_Bake_Diffuse", text="Base Color", icon="BLANK1")
                                        row.prop(context.active_object, "scm_UI_Bake_Roughness", text="Roughness", icon="BLANK1")
                                        row.prop(context.active_object, "scm_UI_Bake_Alpha", text="Alpha", icon="BLANK1")
                                        row = box.row()
                                        row.label(text="", icon="BLANK1")
                                        row.prop(context.active_object, "scm_UI_Bake_AO", text="Ambient Occlusion", icon="BLANK1")
                                        row.prop(context.active_object, "scm_UI_Bake_Emissive", text="Emissive (WIP)", icon="BLANK1")
                                        row.prop(context.active_object, "scm_UI_Bake_Cavity", text="Cavity", icon="BLANK1")
                                        row = box.row()
                                        row.label(text="", icon="BLANK1")
                                        row.prop(context.active_object, "scm_UI_Bake_Normal", text="Normal - Open GL", icon="BLANK1")
                                        row.prop(context.active_object, "scm_UI_Bake_Normal_UE", text="Normal - Direct X", icon="BLANK1")
                                        # row.prop(context.active_object, "scm_UI_Bake_UV", text="Uv´s", icon="BLANK1")
                                        row = box.row()
                                        row.label(icon="SYNTAX_OFF")
                                        row.prop(context.active_object, "scm_image_output_name", text="Texture Name")
                                        
                                        row = box.row()
                                        row.label(text="", icon="BLANK1")
                                        if context.active_object.scm_image_output_name != "":
                                            if active_object.scm_UI_Bake_Diffuse or active_object.scm_UI_Bake_Roughness or active_object.scm_UI_Bake_AO or active_object.scm_UI_Bake_Normal or active_object.scm_UI_Bake_Normal_UE or active_object.scm_UI_Bake_Emissive or active_object.scm_UI_Bake_Alpha or active_object.scm_UI_Bake_Cavity or active_object.scm_UI_Bake_UV:
                                                if context.view_layer.objects.active.select_get() == True:
                                                    row.enabled = True
                                                    row.active_default= True
                                                    row.scale_y = 2.0
                                                    row.operator("object.scm_bake_output_textures", text="Bake selected Textures", icon="RENDER_STILL")
                                                else:
                                                    row.alignment="CENTER"
                                                    row.alert=True
                                                    row.label(text="Please select object to export textures.")
                                                    row = box.row()
                                                    row.enabled = False
                                                    row.alert =True
                                            else:
                                                row.alignment="CENTER"
                                                row.alert=True
                                                row.label(text="please select one Texture for export")
                                                row = box.row()
                                                row.enabled = False
                                                row.alert =True
                                        else:
                                            row.alignment="CENTER"
                                            row.alert=True
                                            row.label(text="Please enter Texture Name")
                                        row = box.row()
                                    #UI ELEMENTS ON OFF
                                    box = layout.box()
                                    row = box.row()
                                    row.label(text="UI - Elements", icon="SHADERFX")
                                    # row.prop(active_object, "scm_UI_Additional_Image", text="Additional Image", icon="EXPORT")
                                    
                                    row.prop(context.active_object, "scm_UI_TRANSFORM", text="Transform | Rotation | Scale", icon="MOD_DATA_TRANSFER")
                                    row = box.row()
                                    row.prop(active_object, "scm_UI_COLORS", text="Colors", icon="IMAGE_DATA")
                                    row.prop(active_object, "scm_UI_Glossy", text="Roughness", icon="SHADING_RENDERED")

                                    row = box.row()
                                    row.prop(active_object, "scm_UI_Normals", text="Normals", icon="GIZMO")
                                    row.prop(active_object, "scm_UI_Alpha", text="Alpha", icon="IMAGE_RGB_ALPHA")

                                    row = box.row()
                                    row.scale_y = 2.0
                                    row.prop(active_object, "scm_UI_Additional_Effects", text="Extra Shading", icon="OVERLAY")



                                    # row = box.row()
                                    if context.active_object.scm_UI_Additional_Effects == True:
                                        box = layout.box()
                                        row = box.row()

                                        row.label(text="Additional Effects", icon="SHADERFX")
                                        
                                        row.prop(context.scene.render , "engine", text="")
                                        # bpy.data.scenes, "engine", text="Render Engine:",
                                        row = box.row()

                                        row.prop(active_object, "scm_UI_Subsurface_Sheen", text="Subsurface | Sheen", icon="OVERLAY")
                                        row.prop(active_object, "scm_UI_Emission", text="Emission (WIP)", icon="OUTLINER_OB_LIGHT")
                                        row = box.row()

                                        row.prop(active_object, "scm_UI_Dirt", text="Dirt", icon="MOD_NOISE")
                                        row.prop(active_object, "scm_UI_Tear", text="Tearing", icon="MOD_PHYSICS")

                                        
                                        if bpy.context.scene.render.engine == 'CYCLES':
                                            rowEnable = True
                                        else:
                                            rowEnable = False
                                        row = box.row()
                                        row.prop(active_object, "scm_UI_Glitter", text="Glitter", icon="SHADERFX")
                                        row.prop(active_object, "scm_UI_Fuzzle", text="Hair | Fuzzles", icon="CURVES")
                                        row = box.row()
                                        row.active = rowEnable
                                        row.prop(active_object, "scm_UI_Wearing", text="Wearing - (Cycles)", icon="MOD_PHYSICS")
                                        row.prop(active_object, "scm_UI_Velvet", text="Velvet - (Cycles)", icon="PROP_OFF")
                                        row = box.row()
                                        row.active = rowEnable
                                        row.prop(active_object, "scm_UI_Dispersion", text="Dispersion | Holographic - (Cycles)", icon="MOD_SHRINKWRAP")
                                        # row = box.row()
                                        row.prop(active_object, "scm_UI_Backface", text="Backface | Inside - (Cycles)", icon="FACESEL")
                                            
                                        if context.active_object.scm_UI_Emission == True:
                                            box = layout.box()
                                            row = box.row()
                                            row.alert = True
                                            row.label(text="Emission (WIP)",icon="LIGHT_SUN")
                                            row = box.row()
                                            row.label(text="", icon="IPO_LINEAR")
                                            row.prop(simplyMaterial.node_tree.nodes["SM_Emission_Intensity"].outputs[0], "default_value",text="Emission Intensity")
                                            row.prop(simplyMaterial.node_tree.nodes["SM_Emission_Contrast"].outputs[0], "default_value",text="Contrast")
                                            row.prop(simplyMaterial.node_tree.nodes["SCM_Emission_Invert"], "mute",text="", icon="IMAGE_ALPHA")
                                            row = box.row()
                                            row.label(text="", icon="OPTIONS")
                                            row.prop(simplyMaterial.node_tree.nodes["SCM_Emission_HueSat"].inputs[0], "default_value", text="Hue", icon="BRUSHES_ALL")
                                            row.prop(simplyMaterial.node_tree.nodes["SCM_Emission_HueSat"].inputs[1], "default_value", text="Saturation", icon="BRUSHES_ALL")
                                            row.prop(simplyMaterial.node_tree.nodes["SCM_Emission_HueSat"].inputs[2], "default_value", text="Value", icon="BRUSHES_ALL")
                                            #EMISSION COLOR RAMP
                                            row = box.row()
                                            emissive_ColorRamp = node_tree.nodes["SCM_Emission_ColorRamp"]
                                            col = box.column(align=True)
                                            col.template_color_ramp(emissive_ColorRamp, "color_ramp", expand = True)   

                                        if context.active_object.scm_UI_Dispersion == True:
                                            box = layout.box()
                                            row = box.row()
                                            row.label(text="Dispersion | Holographic", icon="IMAGE_DATA")
                                            row = box.row()
                                                                            
                                            row.prop(simplyMaterial.node_tree.nodes["SCM_Mix_Dispersion"], "mute", text="", icon="HIDE_OFF", invert_checkbox=True)
                                            row.prop(simplyMaterial.node_tree.nodes["SCM_Dispersion_Balance"].outputs[0],"default_value", text="Mix Intensity", icon="DRIVER_ROTATIONAL_DIFFERENCE")
                                            row = box.row()

                                            row.label(text="", icon="SHAPEKEY_DATA")
                                            row.prop(simplyMaterial.node_tree.nodes["SCM_Dispersion"].outputs[0],"default_value", text="Dispersion | Holographic - Intensity", icon="DRIVER_ROTATIONAL_DIFFERENCE")
                                            row = box.row()

                                            row.label(text="", icon="SHADING_RENDERED")
                                            row.prop(simplyMaterial.node_tree.nodes["SCM_Dispersion_Roughness"].outputs[0],"default_value", text="Roughness", icon="DRIVER_ROTATIONAL_DIFFERENCE")
                                            row.prop(simplyMaterial.node_tree.nodes["SCM_Dispersion_IOR"].outputs[0],"default_value", text="IOR", icon="DRIVER_ROTATIONAL_DIFFERENCE")
                                            row = box.row()

                                            row.label(text="", icon="ORIENTATION_NORMAL")
                                            row.prop(simplyMaterial.node_tree.nodes["SC_Global_Normal_Intensity"].outputs[0], "default_value", text="Global Normal")

                                        if context.active_object.scm_UI_Velvet == True:
                                            
                                            box = layout.box()
                                            row = box.row()

                                            row.label(text="Velvet", icon="IMAGE_DATA")
                                            row = box.row()
                                            
                                            row.prop(simplyMaterial.node_tree.nodes["SCM_Mix_Velvet"], "mute", text="", icon="HIDE_OFF", invert_checkbox=True)
                                            row.prop(simplyMaterial.node_tree.nodes["SCM_Velvet_Sigma"].outputs[0],"default_value", text="Sigma", icon="DRIVER_ROTATIONAL_DIFFERENCE")
                                            row.prop(simplyMaterial.node_tree.nodes["SCM_Velvet_Balance"].outputs[0],"default_value", text="Mix Velvet", icon="DRIVER_ROTATIONAL_DIFFERENCE")

                                        if context.active_object.scm_UI_Backface == True:
                                            box = layout.box()
                                            row = box.row()

                                            row.label(text="Backface | Inside", icon="FACESEL")
                                            row.label(text="", icon="BLANK1")
                                            row.prop(simplyMaterial.node_tree.nodes["SM_Backface_Color_Mix"], "blend_type", text="")
                                            row.prop(simplyMaterial.node_tree.nodes["SM_Backface_Color"].outputs[0], "default_value", text="", icon="HIDE_OFF", invert_checkbox=True)
                                            row = box.row()
                                            row.prop(simplyMaterial.node_tree.nodes["SM_Backface_Shader"], "mute", text="", icon="HIDE_OFF", invert_checkbox=True)
                                            row.prop(simplyMaterial.node_tree.nodes["SM_Backface_Color_Mix_Intensity"].outputs[0],"default_value", text="Mix Intensity", icon="DRIVER_ROTATIONAL_DIFFERENCE")
                                            row = box.row() 
                                            
                                            row.label(text="Base Color", icon="SELECT_INTERSECT")
                                            row.prop(simplyMaterial.node_tree.nodes["SM_Backface_Color_Brightness"].outputs[0],"default_value", text="Intensity", icon="DRIVER_ROTATIONAL_DIFFERENCE")
                                            row = box.row()
                                            row.prop(simplyMaterial.node_tree.nodes["SM_Backface_Color_Saturation"].outputs[0],"default_value", text="Saturation", icon="DRIVER_ROTATIONAL_DIFFERENCE")
                                            row.prop(simplyMaterial.node_tree.nodes["SM_Backface_Color_Hue"].outputs[0],"default_value", text="Hue", icon="DRIVER_ROTATIONAL_DIFFERENCE")

                                        # SUBSURFACE SETTINGS
                                        if active_object.scm_UI_Subsurface_Sheen == True:
                                            box = layout.box()
                                            row = box.row()

                                            row.label(text="Sub-Surface | Sheen- Fresnel | Transmission", icon="SHADERFX")
                                            row = box.row()

                                            # row.label(text="", icon="OVERLAY")
                                            
                                            row.prop(simplyMaterial.node_tree.nodes["SM_Subsurface_Invert"], "mute", text="", icon="IMAGE_ALPHA", invert_checkbox=True)
                                            if simplyMaterial.node_tree.nodes["SC_SubSurface_Intensity"].outputs[0].default_value <0:
                                                row.alert =True

                                            row.prop(simplyMaterial.node_tree.nodes["SC_SubSurface_Intensity"].outputs[0], "default_value", text="Sub-Surface")
                                            row.prop(simplyMaterial.node_tree.nodes["SM Subsurface Blend Mode"], "blend_type", text="")

                                            # SHEEN FRESNEL SETTINGS

                                            row = box.row()
                                            row.label(text="", icon="ANTIALIASED")
                                            row.prop(simplyMaterial.node_tree.nodes["SM_Sheen_Fresnel_Value"].outputs[0], "default_value", text="Fresnel")
                                            row.prop(simplyMaterial.node_tree.nodes["SM_Sheen_Fresnel_Tint"].outputs[0], "default_value", text="Tint")


                                            # TRANSMISSION
                                            row = box.row()
                                            row.label(text="", icon="XRAY")
                                            row.prop(simplyMaterial.node_tree.nodes["SM_Principled_BSDF"].inputs[17], "default_value", text="Transmission")
                                            row.prop(simplyMaterial.node_tree.nodes["SM_Principled_BSDF"].inputs[18], "default_value", text="Roughness")
                                            row = box.row()

                                        #TOOL SETTINGS
                                        if active_object.scm_UI_Dirt == True:
                                            box = layout.box()
                                            row = box.row()
                                            # DIRT
                                            row.label(text="Dirt", icon="MOD_NOISE")
                                            row.label(text="", icon="BLANK1")
                                            row.prop(simplyMaterial.node_tree.nodes["SM_Dirt_Node"].node_tree.nodes["Dirt_Intensity"].outputs[0], "default_value", text="Intensity")
                                            row = box.row()

                                            row.label(text="", icon="FULLSCREEN_ENTER")
                                            row.prop(simplyMaterial.node_tree.nodes["SM_Dirt_Node"].node_tree.nodes["Dirt_Noise"].inputs[2], "default_value", text="Scale")
                                            row.prop(simplyMaterial.node_tree.nodes["SM_Dirt_Node"].node_tree.nodes["Dirt_Noise"].inputs[3], "default_value", text="Detail")
                                            
                                            row = box.row()
                                            row.label(text="", icon="SHADERFX")
                                            row.prop(simplyMaterial.node_tree.nodes["SM_Dirt_Node"].node_tree.nodes["Dirt_Noise"].inputs[4], "default_value", text="Noise")
                                            row.prop(simplyMaterial.node_tree.nodes["SM_Dirt_Node"].node_tree.nodes["Dirt_Noise"].inputs[5], "default_value", text="Distrotion")
                                            #DIRT COLOR RAMP
                                            row = box.row()
                                            dirt_ColorRamp = node_tree.nodes["SM_Dirt_Node"].node_tree.nodes["SM_Dirt_ColorRamp"]
                                            col = box.column(align=True)
                                            col.template_color_ramp(dirt_ColorRamp, "color_ramp", expand = True)   

                                        # TEARING UI
                                        if active_object.scm_UI_Tear == True:
                                            box = layout.box()
                                            row = box.row()

                                            # TEAR
                                            row.label(text="Tearing", icon="MOD_PHYSICS")
                                            row.label(text="", icon="BLANK1")
                                            row.prop(simplyMaterial.node_tree.nodes["SM_Tearing_Node"].node_tree.nodes["Tear_Intensity_Global"].outputs[0], "default_value", text="Tear Intensity")

                                            row = box.row()
                                            row.scale_y = 2.0
                                            row.prop(simplyMaterial.node_tree.nodes["SM_Tearing_Node"].node_tree.nodes["SCM_Tearing_Mix_Texture_To_Noise"], "mute", text="", icon="HIDE_OFF", invert_checkbox=True)
                                            row.operator("object.scm_switch_tear_texture_paint", text="Texture Paint", icon="BRUSHES_ALL")
                                            row.prop(context.active_object, "scm_UI_Tear_Procedural", text="Procedural Settings", icon="TOOL_SETTINGS")
                                            row = box.row()

                                            row.label(text="", icon="BLANK1")
                                            row.prop(simplyMaterial.node_tree.nodes["SM_Tearing_Node"].node_tree.nodes["SCM_Tearing_Mix_Noise_Paint"].outputs[0], "default_value", text="Mix Texture Paint with Procedural")
                                            row = box.row()
                                            
                                            if context.active_object.scm_UI_Tear_Paint == True:
                                                pass

                                            if context.active_object.scm_UI_Tear_Procedural == True:
                                                row = box.row()
                                                row.label(text="", icon="NODE_TEXTURE")
                                                row.prop(simplyMaterial.node_tree.nodes["SCM_Tear_Noise"].inputs[2], "default_value", text="Tear Scale")

                                                row = box.row()
                                                row.label(text="",icon="LIBRARY_DATA_BROKEN")

                                                row.prop(simplyMaterial.node_tree.nodes["SCM_Tear_Noise"].inputs[4], "default_value", text="Tear Detail")
                                                row.prop(simplyMaterial.node_tree.nodes["SM_Tearing_Node"].node_tree.nodes["Fiber_Normal_Intensity"].outputs[0], "default_value", text="Tear Normal")
                                                row = box.row()
                                                
                                                row = box.row()
                                                row.label(text="", icon="OUTLINER_DATA_LIGHTPROBE")
                                                row.prop(simplyMaterial.node_tree.nodes["SM_Tearing_Node"].node_tree.nodes["Tear_Fiber_Intensity"].outputs[0], "default_value", text="Fiber Intensity")
                                                row.prop(simplyMaterial.node_tree.nodes["SM_Tearing_Node"].node_tree.nodes["Tear_Fiber_Scale"].outputs[0], "default_value", text="Fiber Scale")

                                                row = box.row()

                                                row.label(text="", icon="FULLSCREEN_ENTER")
                                                row.prop(simplyMaterial.node_tree.nodes["SM_Tearing_Node"].node_tree.nodes["Tear_Fringle_intensity"].outputs[0], "default_value", text="Fringle Intensity")
                                                row.prop(simplyMaterial.node_tree.nodes["SM_Tearing_Node"].node_tree.nodes["Tear_Fiber_Distort"].outputs[0], "default_value", text="Fiber Distortion")
                                                row = box.row()

                                            row.label(text="Tearing Color", icon="RESTRICT_COLOR_OFF")
                                            row = box.row()

                                            row.prop(simplyMaterial.node_tree.nodes["SM_Tearing_Node"].node_tree.nodes["RGB_TEAR_COLOR"].outputs[0], "default_value", text="")
                                            row = box.row()

                                            #TEARING COLOR RAMP
                                            row.label(text="Tearing Contrast Ramp", icon="SEQ_HISTOGRAM")
                                            tear_ColorRamp = node_tree.nodes["SM_Tearing_Node"].node_tree.nodes["Tear_Input_Ramp"]
                                            col = box.column(align=True)
                                            col.template_color_ramp(tear_ColorRamp, "color_ramp", expand = True)           
                                            
                                        # WEARING WIP
                                        if bpy.context.scene.render.engine == 'CYCLES':
                                            if bpy.context.active_object.scm_UI_Wearing == True:
                                                box = layout.box()
                                                row = box.row()
                                                wearing= "Wearing"
                                                wearingIcon = "SORTTIME"

                                                if context.scene.render.engine == "BLENDER_EEVEE" or context.scene.render.engine == "BLENDER_WORKBENCH":
                                                    row = box.row()
                                                    wearing = "Realtime Wearing | Only with Cycles"
                                                    wearingIcon = "ERROR"
                                                    row.alert=True

                                                if context.scene.render.engine == "CYCLES":
                                                    wearing = "Wearing"
                                                    wearingIcon = "SORTTIME"

                                                row.prop(simplyMaterial.node_tree.nodes["SM_Node_Group_Wearing"].node_tree.nodes["SCM_Wear_Mix"], "mute", text="", icon="HIDE_OFF", invert_checkbox=True)
                                                row.label(text=wearing)
                                                # row.label(text="", icon="BLANK1")
                                                row.operator("scene.sm_preview_cavity_bake",text="Preview Cavity", icon="HIDE_OFF")
                                                row = box.row()

                                                if context.scene.scm_UI_PreviewForCavityBake == True:
                                                    #WAERING BAKE CAVITY COLOR RAMP
                                                    row = box.row()
                                                    row.label(text="Bake Cavity Mask Adjustment", icon="SEQ_HISTOGRAM")
                                                    tear_bake_ColorRamp = node_tree.nodes["SM_Wearing_Ramp_For_Bake"]
                                                    col = box.column(align=True)
                                                    col.template_color_ramp(tear_bake_ColorRamp, "color_ramp", expand = True)     
                                                    row = box.row()
                                                
                                                row.prop(simplyMaterial.node_tree.nodes["SM_Wearing_Cavity_Invert"], "mute", text="", icon="IMAGE_ALPHA", invert_checkbox=True)
                                                row.prop(simplyMaterial.node_tree.nodes["SM_Node_Group_Wearing"].node_tree.nodes["SCM_Wear_Intensity"].outputs[0], "default_value", text="Wear Intensity")
                                                row.prop(simplyMaterial.node_tree.nodes["SM_Wearing_Normal_Intensity"].outputs[0], "default_value", text="Normal Intensity")
                                                row = box.row()

                                                tear_ColorRamp = node_tree.nodes["SM_Wearing_Ramp"]
                                                col = box.column(align=True)
                                                col.template_color_ramp(tear_ColorRamp, "color_ramp", expand = True)     

                                                # row = box.row()
                                                # row.label(text="Wear Mix Texture", icon="IMAGE")
                                                # row.prop(simplyMaterial.node_tree.nodes["SCM_Wear_Noise_Addition_Texture"].image, "filepath", text="", icon="IMAGE_BACKGROUND")
                                                # row = box.row()
                                                # row.prop(simplyMaterial.node_tree.nodes["SC_Wear_Noise_Texture_Scale"].outputs[0], "default_value", text="Scaling")
                                                # row = box.row()
                                                # tear_texture_ColorRamp = node_tree.nodes["SCM_Wear_Noise_Addition_Texture_ColorRamp"]
                                                # col = box.column(align=True)
                                                # col.template_color_ramp(tear_texture_ColorRamp, "color_ramp", expand = True)     
                                                # row = box.row()


                                        if context.active_object.scm_UI_Fuzzle == True:
                                            box = layout.box()
                                            row = box.row()
                                            if context.active_object.scm_Fuzzle_Modifier == False:
                                                row.label(text="Hair | Fuzzles", icon="CURVES")
                                                row.operator("object.sm_setup_fuzzle_geo_nodes", text="Create Hair Fuzzles", icon="CURVES")
                                            if context.active_object.scm_Fuzzle_Modifier == True:
                                                simplyFuzzleModifier = context.active_object.modifiers[context.active_object.scm_geoNodes_fuzzle_modifier_name]
                                                simplyFuzzleGeoNodes = simplyFuzzleModifier.node_group
                                                
                                                row.label(text="Hair | Fuzzles", icon="CURVES")
                                                row.prop(simplyFuzzleModifier, "show_in_editmode",text="")
                                                row.prop(simplyFuzzleModifier, "show_viewport",text="")
                                                row.prop(simplyFuzzleModifier, "show_render",text="")
                                                row = box.row()
                                                row.prop(simplyFuzzleGeoNodes.nodes["SM_Fuzz_Node_Collection"].inputs[0],"default_value", text="Collection", icon="OUTLINER_COLLECTION")
                                                row = box.row()
                                                row.prop(simplyFuzzleGeoNodes.nodes["SM_Fuzzle_Material"].inputs[2],"default_value", text="Material", icon="MATERIAL")

                                                if simplyFuzzleGeoNodes.nodes["SM_Fuzz_Node_Collection"].inputs[0].default_value != "":
                                                    #VIEWPORT | RENDER
                                                    row = box.row()
                                                    row.label(text="Viewport | Render", icon="TOOL_SETTINGS")
                                                    row = box.row()
                                                    row.label(text="", icon="RESTRICT_VIEW_OFF")
                                                    row.prop(simplyFuzzleGeoNodes.nodes["SM_Fuzzle_Render_Viewport_Density"].inputs[3],"default_value", text="")
                                                    row.label(text="", icon="RESTRICT_RENDER_OFF")
                                                    row.prop(simplyFuzzleGeoNodes.nodes["SM_Fuzzle_Render_Viewport_Density"].inputs[2],"default_value", text="")
                                                    
                                                    #SCALE
                                                    box = layout.box()
                                                    row = box.row()
                                                    row.label(text="Scale", icon="FULLSCREEN_ENTER")
                                                    # row = box.row()
                                                    row.label(text="", icon="BLANK1")
                                                    row.label(text="", icon="ORIENTATION_GLOBAL")
                                                    row.prop(simplyFuzzleGeoNodes.nodes["SM_Fuzz_Size_Global"].outputs[0],"default_value", text="Global Scale")

                                                    row = box.row()
                                                    row.label(text="", icon="GP_MULTIFRAME_EDITING")
                                                    row.prop(simplyFuzzleGeoNodes.nodes["SM_Fuzz_Node_Normal_Scale"].outputs[0],"default_value", text="1. Fuzzle")
                                                    row.prop(simplyFuzzleGeoNodes.nodes["SM_Fuzz_Node_Distorted_Scale"].outputs[0],"default_value", text="2. Fuzzle")
                                                    row.prop(simplyFuzzleGeoNodes.nodes["SM_Fuzz_Node_Curly_Scale"].outputs[0],"default_value", text="3. Fuzzle")
                                                    row = box.row()
                                                    row.label(text="", icon="SORTSIZE")
                                                    row.prop(simplyFuzzleGeoNodes.nodes["SM_Fuzz_Size_Min"].outputs[0],"default_value", text="Min Scale")
                                                    row.prop(simplyFuzzleGeoNodes.nodes["SM_Fuzz_Size_Max"].outputs[0],"default_value", text="Max Scale")

                                                    # row = box.row()
                                                    #DENSITY
                                                    box = layout.box()
                                                    row = box.row()
                                                    row.label(text="Density", icon="SEQ_HISTOGRAM")
                                                    # row = box.row()
                                                    row.label(text="", icon="BLANK1")
                                                    row.label(text="", icon="OUTLINER_OB_POINTCLOUD")
                                                    row.prop(simplyFuzzleGeoNodes.nodes["SM_Fuzz_Node_Global_Density"].outputs[0],"default_value", text="Global Density Multiplier")
                                                    row = box.row()
                                                    row.label(text="", icon="GP_MULTIFRAME_EDITING")
                                                    row.prop(simplyFuzzleGeoNodes.nodes["SM_Fuzz_Node_Normal_Density"].outputs[0],"default_value", text="1. Fuzzle")
                                                    row.prop(simplyFuzzleGeoNodes.nodes["SM_Fuzz_Node_Distorted_Density"].outputs[0],"default_value", text="2. Fuzzle")
                                                    row.prop(simplyFuzzleGeoNodes.nodes["SM_Fuzz_Node_Curly_Density"].outputs[0],"default_value", text="3. Fuzzle")
                                                    row = box.row()
                                                    row.label(text="", icon="STICKY_UVS_DISABLE")
                                                    row.prop(simplyFuzzleGeoNodes.nodes["SM_Fuzz_Node_Normal"].inputs[3],"default_value", text="1. Seed")
                                                    row.prop(simplyFuzzleGeoNodes.nodes["SM_Fuzz_Node_Distorted"].inputs[3],"default_value", text="2. Seed")
                                                    row.prop(simplyFuzzleGeoNodes.nodes["SM_Fuzz_Node_Curly"].inputs[3],"default_value", text="3. Seed")
                                                    row = box.row()

                                                    #THICKNESS
                                                    box = layout.box()
                                                    row = box.row()
                                                    row.label(text="Curve Thickness", icon="PARTICLE_PATH")
                                                    row.label(text="", icon="BLANK1")
                                                    row.label(text="", icon="PARTICLE_PATH")
                                                    row.prop(simplyFuzzleGeoNodes.nodes["SM_Fuzz_Fill_Caps"],"boolean", text="Fill Caps", icon="BLANK1")
                                                    row = box.row()
                                                    row.label(text="", icon="PARTICLE_POINT")
                                                    row.prop(simplyFuzzleGeoNodes.nodes["SM_Fuzz_Root_Thickness"].outputs[0],"default_value", text="Root")
                                                    row.prop(simplyFuzzleGeoNodes.nodes["SM_Fuzz_Tip_Thickness"].outputs[0],"default_value", text="Tip",icon="PARTICLE_TIP")
                                                    row = box.row()

                                                    #ROTATIONS
                                                    box = layout.box()
                                                    row = box.row()
                                                    row.label(text="Curve Rotations", icon="CON_ROTLIKE")
                                                    row.label(text="", icon="BLANK1")
                                                    row.label(text="", icon="STICKY_UVS_DISABLE")
                                                    row.prop(simplyFuzzleGeoNodes.nodes["SM_Fuzz_Rotation"].inputs[8],"default_value", text="Seed",icon="STICKY_UVS_DISABLE")

                                                    # row.label(text="", icon="BLANK1")
                                                    row = box.row()
                                                    row.label(text="", icon="EMPTY_ARROWS")
                                                    row.prop(simplyFuzzleGeoNodes.nodes["SM_Fuzz_Rotation"].inputs[1],"default_value", text="X, Y, Z",icon="EMPTY_ARROWS")

                                        #GLITTER
                                        if context.active_object.scm_UI_Glitter == True:
                                            box = layout.box()
                                            row = box.row()
                                            
                                            row.label(text="Glitter", icon="SHADERFX")
                                            row.label(text="",icon="BLANK1")
                                            row.prop(simplyMaterial.node_tree.nodes["SM_Glitter_Color_Blend"], "blend_type", text="", icon="HIDE_OFF", invert_checkbox=True)
                                            # row.operator("object.sm_setup_fuzzle_geo_nodes", text="Create Hair Fuzzles", icon="CURVES")
                                            row = box.row()
                                            # row.label(text="", icon="ALIGN_FLUSH")
                                            row.prop(simplyMaterial.node_tree.nodes["SCM_Glitter_Normal_Mix"], "mute", text="", icon="HIDE_OFF", invert_checkbox=True)
                                            row.prop(simplyMaterial.node_tree.nodes["SM_Glitter_Intensity"].outputs[0], "default_value", text="Intensity")
                                            row.prop(simplyMaterial.node_tree.nodes["SM_Glitter_Scale"].outputs[0], "default_value", text="Scale")
                                            row = box.row()
                                            row.label(text="", icon="SETTINGS")
                                            row.prop(simplyMaterial.node_tree.nodes["SM_Glitter_Smooth"].outputs[0], "default_value", text="Smoothness")
                                            row.prop(simplyMaterial.node_tree.nodes["SM_Glitter_Voronoi"].inputs[5], "default_value", text="Random", icon="BLANK1")
                                            
                                            row = box.row()
                                            # row.label(text="", icon="SELECT_SUBTRACT")
                                            row.prop(simplyMaterial.node_tree.nodes["SM_Mask_Base_for_Glitter_Invert"], "mute", text="", icon="IMAGE_ALPHA", invert_checkbox=True)
                                            row.prop(simplyMaterial.node_tree.nodes["SM_Mask_Base_for_Glitter"], "mute", text="Base as Mask", icon="SELECT_SUBTRACT", invert_checkbox=True)
                                            row = box.row()
                                            row.label(text="Roughness | Metallic", icon="SHADING_RENDERED")
                                            row.label(text="",icon="BLANK1")
                                            row.prop(simplyMaterial.node_tree.nodes["SM_Glitter_Roughness_Mix"].outputs[0], "default_value", text="Mix to Roughness", icon="ADD")
                                            row = box.row()
                                            row.label(text="", icon="SHADING_RENDERED")
                                            row.prop(simplyMaterial.node_tree.nodes["SM_Roughness"].outputs[0], "default_value", text="Roughness", icon="IMAGE_ALPHA", invert_checkbox=True)
                                            row.prop(simplyMaterial.node_tree.nodes["SM_Principled_BSDF"].inputs[6], "default_value", text="Metallic")
                                            #GLITTER COLOR RAMP
                                            row = box.row()
                                            glitter_ColorRamp = simplyMaterial.node_tree.nodes["SM_Glitter_Ramp"]
                                            col = box.column(align=True)
                                            col.template_color_ramp(glitter_ColorRamp, "color_ramp", expand = True)  

                                    if context.active_object.scm_UI_TRANSFORM == True:
                                        box = layout.box()
                                        row = box.row()

                                        row.label(text="Transform | Rotation | Scale ", icon="IMAGE_DATA")
                                        box = layout.box()
                                        row = box.row()

                                        row.label(text="Base", icon="IMAGE_DATA")
                                        # row = box.row()
                                        row.label(text="", icon="BLANK1")
                                        row.prop(simplyMaterial.node_tree.nodes["SC_Mapping_Base"].node_tree.nodes["SC_Base_Vector_Scale_Add"].inputs[1],"default_value", text="X-Scale", icon="MOD_DATA_TRANSFER", index=0)
                                        row.prop(simplyMaterial.node_tree.nodes["SC_Mapping_Base"].node_tree.nodes["SC_Base_Vector_Scale_Add"].inputs[1],"default_value", text="Y-Scale", icon="MOD_DATA_TRANSFER", index=1)
                                        row = box.row()
                                        
                                        row.label(text="", icon="DRIVER_ROTATIONAL_DIFFERENCE")
                                        row.prop(simplyMaterial.node_tree.nodes["SC_Mapping_Base"].node_tree.nodes["SM_Rotate"].inputs[3],"default_value", text="°", icon="DRIVER_ROTATIONAL_DIFFERENCE")
                                        row.prop(simplyMaterial.node_tree.nodes["SC_Mapping_Base"].inputs[1],"default_value", text="X-Pos.", icon="MOD_DATA_TRANSFER", index=0)
                                        row.prop(simplyMaterial.node_tree.nodes["SC_Mapping_Base"].inputs[1],"default_value", text="Y-Pos.", icon="MOD_DATA_TRANSFER", index=1)

                                        box = layout.box()
                                        row = box.row()
                                        row.label(text="Micro", icon="ALIGN_FLUSH")
                                        # row.label(text="Scale", icon="FULLSCREEN_ENTER")
                                        row.label(text="", icon="BLANK1")
                                        row.prop(simplyMaterial.node_tree.nodes["SC Mapping_Micro"].node_tree.nodes["SC_Micro_Vector_Scale_Add"].inputs[1],"default_value", text="X-Scale", icon="MOD_DATA_TRANSFER", index=0)
                                        row.prop(simplyMaterial.node_tree.nodes["SC Mapping_Micro"].node_tree.nodes["SC_Micro_Vector_Scale_Add"].inputs[1],"default_value", text="Y-Scale", icon="MOD_DATA_TRANSFER", index=1)
                                        row = box.row()
                                        row.label(text="", icon="DRIVER_ROTATIONAL_DIFFERENCE")
                                        row.prop(simplyMaterial.node_tree.nodes["SC Mapping_Micro"].node_tree.nodes["SM_Rotate"].inputs[3],"default_value", text="° ", icon="DRIVER_ROTATIONAL_DIFFERENCE")
                                        row.prop(simplyMaterial.node_tree.nodes["SC Mapping_Micro"].inputs[1],"default_value", text="X-Pos.", icon="MOD_DATA_TRANSFER", index=0)
                                        row.prop(simplyMaterial.node_tree.nodes["SC Mapping_Micro"].inputs[1],"default_value", text="Y-Pos.", icon="MOD_DATA_TRANSFER", index=1)
                                        row = box.row()

                                        box = layout.box()
                                        row = box.row()
                                        row.label(text="Overlay", icon="UV")
                                        # row.label(text="Scale", icon="FULLSCREEN_ENTER")
                                        row.label(text="", icon="BLANK1")
                                        row.prop(simplyMaterial.node_tree.nodes["SC Mapping_Overlay"].node_tree.nodes["SC_Overlay_Vector_Scale_Add"].inputs[1],"default_value", text="X-Scale", icon="MOD_DATA_TRANSFER", index=0)
                                        row.prop(simplyMaterial.node_tree.nodes["SC Mapping_Overlay"].node_tree.nodes["SC_Overlay_Vector_Scale_Add"].inputs[1],"default_value", text="Y-Scale", icon="MOD_DATA_TRANSFER", index=1)
                                        row = box.row()
                                        row.label(text="", icon="DRIVER_ROTATIONAL_DIFFERENCE")
                                        row.prop(simplyMaterial.node_tree.nodes["SC Mapping_Overlay"].node_tree.nodes["SM_Rotate"].inputs[3],"default_value", text="°", icon="DRIVER_ROTATIONAL_DIFFERENCE")
                                        row.prop(simplyMaterial.node_tree.nodes["SC Mapping_Overlay"].inputs[1],"default_value", text="X-Pos.", icon="MOD_DATA_TRANSFER", index=0)
                                        row.prop(simplyMaterial.node_tree.nodes["SC Mapping_Overlay"].inputs[1],"default_value", text="Y-Pos.", icon="MOD_DATA_TRANSFER", index=1)

                                    

                        if simplyMaterial is not None:
                            # NORMAL SETTINGS
                            if active_object.scm_UI_Normals == True:
                                box = layout.box()
                                row = box.row()

                                renderEngine= "Render Engine | "+ context.scene.render.engine
                                row.label(text=renderEngine, icon="SNAP_NORMAL")  
                                row.prop(context.scene.render, "engine", text="", icon="SETTINGS")
                                row = box.row()

                                iconHide = ""
                                if bpy.context.scene.render.engine == 'CYCLES':
                                    row.active_default = True   

                                if simplyMaterial.node_tree.nodes["SM_Displacement_Node"].mute == False:
                                    iconHide="HIDE_OFF"
                                    row.prop(simplyMaterial.node_tree.nodes["SM_Displacement_Node"], "mute", text="", icon=iconHide)
                                    op = row.operator("scene.scm_cycles_experimental_subdivision", text="Cycles Displacement", icon="PREFERENCES")

                                if simplyMaterial.node_tree.nodes["SM_Displacement_Node"].mute == True:
                                    row.alert = True
                                    iconHide = "HIDE_ON"
                                    row.prop(simplyMaterial.node_tree.nodes["SM_Displacement_Node"], "mute", text="", icon=iconHide)
                                    op = row.operator("scene.scm_cycles_experimental_subdivision", text="Cycles Displacement", icon="PREFERENCES")
                                    row = box.row()
                                # CYCLES DISPLACEMENT SETTINGS
                                if context.scene.render.engine == "CYCLES":
                                    if simplyMaterial.node_tree.nodes["SM_Displacement_Node"].mute == False:
                                        row = box.row()
                                        row.scale_y = 2.0
                                        row.label(text="", icon="MOD_DISPLACE")
                                        row.active = True
                                        if simplyMaterial.node_tree.nodes["SC_Global_Displacement_Intensity"].outputs[0].default_value < 0.0:
                                            row.alert = True
                                        if "Subdivision" not in context.active_object.modifiers:
                                            # row.label(text="Please add Subsurface modifier")/
                                            row.operator("object.setup_modifier_for_displacement", text="Setup for Displacement", icon="PLUS")
                                            row = box.row()
                                        # row.label(text="", icon="BLANK1")
                                        row.prop(simplyMaterial.node_tree.nodes["SC_Global_Displacement_Intensity"].outputs[0], "default_value", text="Displacement Intensity")
                                        row.prop(simplyMaterial.node_tree.nodes["SM_Displacement_Invert"], "mute", text="", icon="IMAGE_ALPHA", invert_checkbox=True)
                                        row = box.row()
                                        row.label(text="", icon="BLANK1")
                                        row.prop(context.scene.cycles, "preview_dicing_rate", text="Viewport Density", icon="IMAGE_ALPHA", invert_checkbox=True)
                                        row.prop(context.scene.cycles, "dicing_rate", text="Render Density", icon="IMAGE_ALPHA", invert_checkbox=True)
                                        row = box.row()
                                        
                                        # row.label(text="", icon="UV")
                                        row.prop(simplyMaterial.node_tree.nodes["SCM_Overlay_Displacement_Invert"], "mute", text="", icon="IMAGE_ALPHA", invert_checkbox=True)
                                        row.prop(simplyMaterial.node_tree.nodes["SCM_Custom_Displacement_Intensity"].outputs[0], "default_value", text="Overlay Mix Intensity")
                                        row.prop(simplyMaterial.node_tree.nodes["SCM_Overlay_Displacement_Mix"], "blend_type", text="")

                                        #DISPLACEMENT COLOR RAMP
                                        row = box.row()
                                        displacementTexture_ColorRamp = node_tree.nodes["SM_Displacement_Ramp"]
                                        col = box.column(align=True)
                                        col.template_color_ramp(displacementTexture_ColorRamp, "color_ramp", expand=True)
                                        row = box.row()
                                        if simplyMaterial.node_tree.nodes["SC_Global_Normal_Intensity"].outputs[0].default_value < 0.0:
                                            row.alert =True
                                            row.scale_y = 2.0

                                            row = box.row()
                                # GLOBAL NORMAL INTENSITY
                                box = layout.box()
                                row = box.row()
                                row.label(text="Normal Settings", icon="SNAP_NORMAL")
                                row.label(text="", icon="BLANK1")
                                row.prop(context.active_object, "scm_UI_normal_custom_active", text="Custom Normal Map Texture", icon="FILE_IMAGE", invert_checkbox=False)

                                if context.active_object.scm_UI_normal_custom_active == True:
                                    row = box.row()
                                    # row = box.row()
                                    row.prop(simplyMaterial.node_tree.nodes["SCM_Custom_Normal_Texture"].image, "filepath", text="", icon="IMAGE_BACKGROUND")
                                    row = box.row()
                                    
                                    row.label(text="", icon="IMAGE_DATA")
                                    row.prop(simplyMaterial.node_tree.nodes["SC_CustomNormal_Texture_Scale"].outputs[0], "default_value", text="Custom Scale")
                                    row.prop(simplyMaterial.node_tree.nodes["SCM_Custom_Normal_Intensity"].outputs[0], "default_value", text="Custom Intensity", invert_checkbox=True, icon="MOD_MASK")
                                    row = box.row()
            
                                    # MICRO NORMAL
                                    row = box.row()
                                    row.label(text="", icon="ALIGN_FLUSH")
                                    if simplyMaterial.node_tree.nodes["SC Micro Texture Intensity"].outputs[0].default_value < 0.0:
                                        row.alert =True 
                                    row.prop(simplyMaterial.node_tree.nodes["SCM_Custom_Normal_Micro_Mix_Intensity"].outputs[0], "default_value", text="Micro Normal")
                                    row.prop(simplyMaterial.node_tree.nodes["SM_Micro_Invert"], "mute", text="", icon="IMAGE_ALPHA", invert_checkbox=True)

                                # if context.active_object.scm_UI_normal_custom_active == False:
                                row = box.row()
                                row.label(text="", icon="ORIENTATION_NORMAL")
                                row.prop(simplyMaterial.node_tree.nodes["SC_Global_Normal_Intensity"].outputs[0], "default_value", text="Global Normal")
                                row.prop(simplyMaterial.node_tree.nodes['SM_Normal_Global'], "invert", text="", icon="IMAGE_ALPHA")
                                
                                # BASE NORMAL
                                row = box.row()
                                row.label(text="", icon="IMAGE_DATA")
                                if simplyMaterial.node_tree.nodes["SC_Base_Normal_Intensity"].outputs[0].default_value < 0.0:
                                    row.alert =True 
                                row.prop(simplyMaterial.node_tree.nodes["SC_Base_Normal_Intensity"].outputs[0], "default_value", text="Base Normal")
                                row.prop(simplyMaterial.node_tree.nodes["SM_Base_Normal_Invert"], "mute", text="", icon="IMAGE_ALPHA", invert_checkbox=True)
                                row = box.row()
                                row.label(text="", icon="SELECT_DIFFERENCE")
                                row.prop(simplyMaterial.node_tree.nodes["SCM_Mask_Base_to_Micro"], "mute", text="Base as Mask for Micro", invert_checkbox=True, icon="MOD_MASK")
                                row.prop(simplyMaterial.node_tree.nodes["SCM_INVERT_Mask_Base_to_Micro"], "mute", text="", icon="IMAGE_ALPHA")
                                
                            
                                # MICRO NORMAL
                                row = box.row()
                                # row.label(text="", icon="ALIGN_FLUSH")
                                row.prop(simplyMaterial.node_tree.nodes["SM_Micro_Invert"], "mute", text="", icon="IMAGE_ALPHA", invert_checkbox=True)

                                if simplyMaterial.node_tree.nodes["SC Micro Texture Intensity"].outputs[0].default_value < 0.0:
                                    row.alert =True 

                                row.prop(simplyMaterial.node_tree.nodes["SC Micro Texture Intensity"].outputs[0], "default_value", text="Micro Normal")
                                row.prop(simplyMaterial.node_tree.nodes["SM_MixNormal_BaseToMicro"], "blend_type", text="", icon="NODE_TEXTURE")                            
                                
                                # OVERLAY NORMAL
                                row = box.row()
                                # row.label(text="", icon="OVERLAY")
                                row.prop(simplyMaterial.node_tree.nodes["SM_Overlay_Invert"], "mute", text="", icon="IMAGE_ALPHA", invert_checkbox=True)

                                if simplyMaterial.node_tree.nodes["SC_Overlay_Normal_Intensity"].outputs[0].default_value < 0.0:
                                    row.alert =True 

                                row.prop(simplyMaterial.node_tree.nodes["SC_Overlay_Normal_Intensity"].outputs[0], "default_value", text="Overlay Normal")
                                row.prop(simplyMaterial.node_tree.nodes["SCM_Overlay_Normal_Mode"], "blend_type", text="", icon="NODE_TEXTURE")
                                
                                row = box.row()
                                row.prop(simplyMaterial.node_tree.nodes["SC_Custom_Texture_Normal_Invert"], "mute", text="", icon="IMAGE_ALPHA", invert_checkbox=True)
                                row.prop(simplyMaterial.node_tree.nodes["SC_Custom_Texture_Normal_Intensity"].outputs[0], "default_value", text="Additional Normal")
                                row.prop(simplyMaterial.node_tree.nodes["SM_MixNormal_Additional_Blend"], "blend_type", text="", icon="NODE_TEXTURE")

                                #DIRT COLOR RAMP
                                row = box.row()
                                base_normal_ColorRamp = node_tree.nodes["Normal_Base_ColorRamp"]
                                col = box.column(align=True)
                                col.template_color_ramp(base_normal_ColorRamp, "color_ramp", expand = True)  

                            # ROUGHNESS SETTINGS
                            if active_object.scm_UI_Glossy == True:
                                box = layout.box()
                                row = box.row()
                                row.label(text="Roughness | Metallic | Specular", icon="NODE_MATERIAL")
                                if context.active_object.scm_UI_roughness_custom_active == True:
                                    row.prop(simplyMaterial.node_tree.nodes["SC_Roughness_Texture_Mix"], "blend_type", text="")
                                row = box.row()
                                row.prop(context.active_object, "scm_UI_roughness_custom_active", text="", icon="HIDE_OFF", invert_checkbox=False)
                                row.label(text="Custom Roughness Texture")

                                row.prop(simplyMaterial.node_tree.nodes["SC_Roughness_Texture"].image, "filepath", text="", icon="IMAGE_BACKGROUND")
                                if context.active_object.scm_UI_roughness_custom_active == True:
                                    # row = box.row()
                                    
                                    #CUSTOM ROUGHNESS TEXTURE COLOR RAMP
                                    row = box.row()
                                    # row.label(text="", icon="BLANK1")
                                    row.prop(simplyMaterial.node_tree.nodes["SC_Roughness_Texture_Invert"], "mute", text="", icon="IMAGE_ALPHA", invert_checkbox=True)
                                    row.prop(simplyMaterial.node_tree.nodes["SC_Roughness_Texture_Scale"].outputs[0], "default_value", text="Scale")
                                    row.prop(simplyMaterial.node_tree.nodes["SC_Roughness_Texture_Intensity"].outputs[0], "default_value", text="Intensity")
                                    row = box.row()
                                    
                                    # row = box.row()
                                    roughnessCustomTexture_ColorRamp = node_tree.nodes["SC_Roughness_Texture_ColorRamp"]
                                    col = box.column(align=True)
                                    col.template_color_ramp(roughnessCustomTexture_ColorRamp, "color_ramp", expand = True) 
                                box = layout.box()
                                # row.prop(simplyMaterial.node_tree.nodes["SM_Roughness_MIX"].outputs[0], "default_value", text="Mix Texture")
                                row = box.row()
                                # row.label(text="", icon="RENDERLAYERS")
                                row.label(text="Blend Roughness", icon="RENDERLAYERS")
                                row.prop(simplyMaterial.node_tree.nodes["SM_Roughness_Blend"], "blend_type", text="")
                                # row.label(text="", icon="RENDERLAYERS")
                                row = box.row()
                                if simplyMaterial.node_tree.nodes["SM_Roughness_MIX"].outputs[0].default_value <0:
                                    row.alert =True
                                row = box.row()
                                row.label(text="", icon="BLANK1")
                                row.prop(simplyMaterial.node_tree.nodes["SM_Roughness_MIX"].outputs[0], "default_value", text="Mix Base Texture")
                                row.prop(simplyMaterial.node_tree.nodes["SM_Roughness_BaseColor_Invert"], "mute", text="", icon="IMAGE_ALPHA", invert_checkbox=True)
                                row = box.row()

                                row.label(text="", icon="SHADING_RENDERED")
                                if simplyMaterial.node_tree.nodes["SM_Roughness"].outputs[0].default_value <0:
                                    row.alert =True

                                row.prop(simplyMaterial.node_tree.nodes["SM_Roughness"].outputs[0], "default_value", text="Roughness")
                                row.prop(simplyMaterial.node_tree.nodes["SM_Roughness_Invert"], "mute", text="", icon="IMAGE_ALPHA")
                                row.prop(simplyMaterial.node_tree.nodes["SM_Principled_BSDF"].inputs[6], "default_value", text="Metallic")
                                row = box.row()

                                row.label(text="", icon="NODE_MATERIAL")
                                row.prop(simplyMaterial.node_tree.nodes["SM_Principled_BSDF"].inputs[7], "default_value", text="Specular")
                                row.label(text="", icon="MATSHADERBALL")
                                row.prop(simplyMaterial.node_tree.nodes["SM_Principled_BSDF"].inputs[8], "default_value", text="Specular Tint")
                                row = box.row()


                                row.label(text="", icon="MATSHADERBALL")
                                row.prop(simplyMaterial.node_tree.nodes["SM_Principled_BSDF"].inputs[14], "default_value", text="Clear Coat | Intensity")
                                row.label(text="", icon="SHADING_TEXTURE")
                                row.prop(simplyMaterial.node_tree.nodes["SM_Principled_BSDF"].inputs[15], "default_value", text="Clear Coat | Roughness")
                                if context.active_object.scm_UI_roughness_custom_active == False:
                                    #ROUGHNESS TEXTURE COLOR RAMP
                                    row = box.row()
                                    roughnessTexture_ColorRamp = node_tree.nodes["SM_Roughness_Color_Ramp"]
                                    col = box.column(align=True)
                                    col.template_color_ramp(roughnessTexture_ColorRamp, "color_ramp", expand = True)    

                            if active_object.scm_UI_Alpha == True:
                                box = layout.box()
                                row = box.row()
                                if simplyMaterial.node_tree.nodes["SCM_Mix_Velvet"].mute == False:
                                    row.alert=True
                                    row.label(text="", icon="ERROR")
                                    row.label(text="Velvet is activated! For correct rendering, please turn off Velvet.")
                                    row = box.row()
                                    row.alert=True
                                    row.label(text="Velvet can influence the rendering and possibly represent transperency incorrectly!")
                                    row = box.row()

                                row.label(text="Alpha", icon="SELECT_SUBTRACT")
                                # row = box.row()
                                # row.label(text="Blend Mode", icon="TEXTURE_DATA")
                                row.label(text="", icon="BLANK1")
                                row.prop(simplyMaterial, "blend_method", text="")
                                row.prop(simplyMaterial, "shadow_method", text="")
                                row = box.row()

                                row.label(text="", icon="NODE_TEXTURE")
                                row.prop(simplyMaterial.node_tree.nodes["SC_Base_Alpha"].outputs[0], "default_value", text="Transparency")                            
                                row = box.row()

                                row.label(text="", icon="IMAGE_DATA")
                                row.prop(simplyMaterial.node_tree.nodes["SC_Base_Alpha_Mix"].outputs[0], "default_value", text="Mix Base to Alpha")
                                row.prop(simplyMaterial.node_tree.nodes["SC_Alpha_Invert"], "mute", text="", icon="IMAGE_ALPHA", invert_checkbox=True)

                                #ALPHA TEXTURE COLOR RAMP
                                row = box.row()
                                row.label(text="Alpha Color Ramp", icon="SELECT_SUBTRACT")
                                alphaTexture_ColorRamp = node_tree.nodes["SM_Alpha_ColorRamp"]
                                col = box.column(align=True)
                                col.template_color_ramp(alphaTexture_ColorRamp, "color_ramp", expand = True)                        

                            #BASE TEXTURE COLOR RAMP
                            if active_object.scm_UI_COLORS == True:
                                box = layout.box()
                                row = box.row()
                                row.label(text="Colors", icon="COLOR")
                                # BLUR NODE FOR BASE COLOR
                                # box = layout.box()
                                row = box.row()
                                row.label(text="Base Color", icon="IMAGE_DATA")
                                row.label(text="", icon="BLANK1")
                                row.prop(simplyMaterial.node_tree.nodes["BaseTexture_ColorRamp"], "mute", text="Use Texture Color", icon="BRUSHES_ALL")
                                row.prop(simplyMaterial.node_tree.nodes["SM_Base_Color_Invert"], "mute", text="", icon="IMAGE_ALPHA", invert_checkbox=True)
                                row = box.row()
                                row.label(text="", icon="MOD_FLUIDSIM")
                                row.prop(simplyMaterial.node_tree.nodes["SCM_Node_Blur_Base"].node_tree.nodes["SCM_Node_Blur_Intensity"].outputs[0], "default_value", text="Blur")
                                row = box.row()
                                
                                row.label(text="", icon="PRESET")
                                row.prop(simplyMaterial.node_tree.nodes["BaseTexture_ColorRamp"], "mute", text="Color Ramp", icon="IPO_EASE_IN_OUT", invert_checkbox=True)
                                # row = box.row()
                                row.prop(simplyMaterial.node_tree.nodes["BaseTexture_HueSatValue"], "mute", text="Hue Sat Val", icon="MOD_HUE_SATURATION", invert_checkbox=True)
                                row = box.row()
                                if simplyMaterial.node_tree.nodes["BaseTexture_HueSatValue"].mute == False:
                                    row.label(text="", icon="BLANK1")
                                    row.prop(simplyMaterial.node_tree.nodes["BaseTexture_HueSatValue"].inputs[0], "default_value", text="Hue", icon="BRUSHES_ALL")
                                    row.prop(simplyMaterial.node_tree.nodes["BaseTexture_HueSatValue"].inputs[1], "default_value", text="Saturation", icon="BRUSHES_ALL")
                                    row.prop(simplyMaterial.node_tree.nodes["BaseTexture_HueSatValue"].inputs[2], "default_value", text="Value", icon="BRUSHES_ALL")
                                
                                if simplyMaterial.node_tree.nodes["BaseTexture_ColorRamp"].mute == False:
                                    baseTexture_ColorRamp = node_tree.nodes["BaseTexture_ColorRamp"]
                                    col = box.column(align=True)
                                    col.template_color_ramp(baseTexture_ColorRamp, "color_ramp", expand = True)

                            if active_object.scm_UI_COLORS == True:
                                if simplyMaterial.node_tree.nodes['SC_Micro_Texture'].image is not None:
                                    #MICRO TEXTURE COLOR RAMP 
                                    box = layout.box()
                                    row = box.row()
                                    row.label(text="Micro Color Ramp", icon="ALIGN_FLUSH")
                                    row.label(text="", icon="BLANK1")
                                    row.prop(simplyMaterial.node_tree.nodes["SM_Mix_Micro_to_Base_Intensity"].outputs[0], "default_value", text="Mix to Base", icon="IMAGE_ZDEPTH")
                                    row.prop(simplyMaterial.node_tree.nodes["SM_Micro_Invert"], "mute", text="", icon="IMAGE_ALPHA")
                                    microTexture_ColorRamp = node_tree.nodes["SM_Micro_Color"]
                                    col = box.column(align=True)
                                    col.template_color_ramp(microTexture_ColorRamp, "color_ramp", expand = True)

                            if active_object.scm_UI_COLORS == True:
                                if simplyMaterial.node_tree.nodes['SC_Overlay_Texture'].image is not None:
                                    #OVERLAY TEXTURE COLOR RAMP 
                                    box = layout.box()
                                    row = box.row()
                                    row.label(text="Overlay Color Ramp", icon="UV")
                                    row.label(text="", icon="BLANK1")
                                    row.prop(simplyMaterial.node_tree.nodes["SM_Mix_Overlay_to_Base_Intensity"].outputs[0], "default_value", text="Mix to Base", icon="IMAGE_ZDEPTH")
                                    row.prop(simplyMaterial.node_tree.nodes["SM_Overlay_Invert"], "mute", text="", icon="IMAGE_ALPHA", invert_checkbox=True)
                                    overlayTexture_ColorRamp = node_tree.nodes["SM_Overlay_ColorRamp"]
                                    col = box.column(align=True)
                                    col.template_color_ramp(overlayTexture_ColorRamp, "color_ramp", expand = True)
 
                elif context.mode == "EDIT":
                    # Edit mode
                    pass
        
        if context.mode == "OBJECT":
            box = layout.box()
            row = box.row()
            # row.scale_y = 2.0
            row.label(text="", icon="FUND")
            row.prop(context.scene ,"scm_UI_Support", text="Simply Support", icon_value=custom_icons["simplyStartIcon"].icon_id)
            row.label(text="", icon="FUND")
            if context.scene.scm_UI_Support == True:
                row = box.row()
                op = row.operator("scene.open_urls", text="HDRi Maker - by Andrew D.", icon="MAT_SPHERE_SKY")
                op.mode = "MATS"    
                op = row.operator("scene.open_urls", text="Extreme Pbr Nexus - by Andrew D.", icon="MATERIAL")
                op.mode = "HDRI"            
                row = box.row()
                op = row.operator("scene.open_urls", text="Auto-Rig-Pro - by Artell", icon="OUTLINER_OB_ARMATURE")
                op.mode = "RIGS"
                op = row.operator("scene.open_urls", text="Voxel Heat Diffuse Skinning - by Mesh Online", icon="OUTLINER_OB_ARMATURE")
                op.mode = "VOXELHEAT"
                row = box.row()
                op = row.operator("scene.open_urls", text="Legendary Chipp Walters Addons", icon="SHADERFX")
                op.mode = "CHIPP"            
                op = row.operator("scene.open_urls", text="K-Cycles - by Eric K.", icon="RESTRICT_RENDER_OFF")
                op.mode = "KCYCLES"
                row = box.row()
                op = row.operator("scene.open_urls", text="Human Generator v3 - by Oliver J. P", icon="ARMATURE_DATA")
                op.mode = "HUMANGEN"            
                op = row.operator("scene.open_urls", text="Epic Addons - by Xane Graphics.", icon="MOD_CLOTH")
                op.mode = "CLOTHSMOTION"  
                row = box.row()
                row.scale_y = 2.0  
                row.label(text="", icon="FUND")
                op = row.operator("scene.open_urls", text="Simply Addons - by Vjaceslav Tissen", icon_value=custom_icons["simplyStartIcon"].icon_id)
                op.mode = "SIMPLYADDONS"
                row.label(text="", icon="FUND")
                box = layout.box()
                row = box.row()
                               
                row.label(text="Customer Support", icon="USER")
                row.label(text="", icon="USER")
                row = box.row()
                
                row.label(text="", icon="HELP")
                op = row.operator("scene.open_urls", text="Help me!", icon_value=custom_icons["simplyStartIcon"].icon_id)
                op.mode = "CUSTOMERSUPPORT"
                row.label(text="", icon="HELP")


def switchCustomNormal(self, context):
    if context.active_object.scm_UI_normal_custom_active == False:
        simplyMaterial =bpy.data.materials[context.active_object.material_slots[context.active_object.material_slots[context.active_object.scm_materialSlot].name].name]
        simplyMaterial.node_tree.nodes["SCM_Custom_Normal_Mix"].mute = True
    if context.active_object.scm_UI_normal_custom_active == True:
        simplyMaterial =bpy.data.materials[context.active_object.material_slots[context.active_object.material_slots[context.active_object.scm_materialSlot].name].name]
        simplyMaterial.node_tree.nodes["SCM_Custom_Normal_Mix"].mute = False

def switchCustomRoughness(self, context):
    if context.active_object.scm_UI_roughness_custom_active == False:
        simplyMaterial =bpy.data.materials[context.active_object.material_slots[context.active_object.material_slots[context.active_object.scm_materialSlot].name].name]
        simplyMaterial.node_tree.nodes["SC_Roughness_Texture_Mix"].mute = True
    if context.active_object.scm_UI_roughness_custom_active == True:
        simplyMaterial =bpy.data.materials[context.active_object.material_slots[context.active_object.material_slots[context.active_object.scm_materialSlot].name].name]
        simplyMaterial.node_tree.nodes["SC_Roughness_Texture_Mix"].mute = False

def switchAdditionalImageTexture(self, context):
    if context.active_object.scm_UI_Additional_Image == False:
        simplyMaterial =bpy.data.materials[context.active_object.material_slots[context.active_object.material_slots[context.active_object.scm_materialSlot].name].name]
        simplyMaterial.node_tree.nodes["SC_Mix_Custom_Base_Color"].mute = True
        simplyMaterial.node_tree.nodes["SM_MixNormal_CustomImage"].mute = True
    if context.active_object.scm_UI_Additional_Image == True:
        simplyMaterial =bpy.data.materials[context.active_object.material_slots[context.active_object.material_slots[context.active_object.scm_materialSlot].name].name]
        simplyMaterial.node_tree.nodes["SC_Mix_Custom_Base_Color"].mute = False
        simplyMaterial.node_tree.nodes["SM_MixNormal_CustomImage"].mute = False
    

def updateBaseIntensity(self, context):
    simplyMaterial =bpy.data.materials[context.active_object.material_slots[context.active_object.material_slots[context.active_object.scm_materialSlot].name].name]
    # print(simplyMaterial)
    simplyMaterial.node_tree.nodes["SC_Base_Color_Intensity"].outputs[0].default_value = context.active_object.scm_Base_Intensity
    
def updateMicroIntensity(self, context):
    simplyMaterial =bpy.data.materials[context.active_object.material_slots[context.active_object.material_slots[context.active_object.scm_materialSlot].name].name]
    # print(simplyMaterial)
    simplyMaterial.node_tree.nodes["SC_Micro_Color_Intensity"].outputs[0].default_value = context.active_object.scm_Micro_Intensity

def updateOverlayIntensity(self, context):
    simplyMaterial =bpy.data.materials[context.active_object.material_slots[context.active_object.material_slots[context.active_object.scm_materialSlot].name].name]
    # print(simplyMaterial)
    simplyMaterial.node_tree.nodes["SC_Overlay_Color_Intensity"].outputs[0].default_value = context.active_object.scm_Overlay_Intensity

def updateAdditionalIntensity(self, context):
    simplyMaterial =bpy.data.materials[context.active_object.material_slots[context.active_object.material_slots[context.active_object.scm_materialSlot].name].name]
    # print(simplyMaterial)
    simplyMaterial.node_tree.nodes["SC_Custom_Texture_Intensity"].outputs[0].default_value = context.active_object.scm_Additional_Intensity

def getCurrentMaterialSlotFromActiveObject(self, context):
    for i, mat in enumerate(context.active_object.material_slots):
        if "SimplyMaterialInfo" in mat.node_tree_nodes:
                self.simplyMaterialSlot = i
                context.active_object.scm_materialSlot = i
def update_image_index(self, context):
    pass

rep_dir_name = Path(__file__).parent.parent.name
addons_path = bpy.utils.user_resource(resource_type="EXTENSIONS",path=rep_dir_name)

def registerIcon():
	import bpy.utils.previews
	global custom_icons

	custom_icons = bpy.utils.previews.new()
	# custom_path = bpy.utils.script_paths()[0]

	# if custom_path is not None:
	# 	addons_path = os.path.join(custom_path, "addons")
	# # print(addons_path)
	# script_path = os.path.join(addons_path,"SimplyMaterial")
	# # print(script_path)
	icons_dir = os.path.join(addons_path, "icons")
	# print(icons_dir)

	custom_icons.load("simplyStartIcon", os.path.join(icons_dir, "simply_icon.png"), 'IMAGE')

def unregisterIcon():
	global custom_icons

def register():
    # Register the panel so it shows up in the sidebar.
    bpy.utils.register_class(SimplyMaterialBasePanel)


def unregister():
    # Unregister the panel to free resources and avoid errors.
    bpy.utils.unregister_class(SimplyMaterialBasePanel)
    # bpy.utils.unregister_class(SimplyMaterialScalePanel)
    